<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.4
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include(dirname(dirname(dirname(__FILE__))).'/includes/404.php');
}

class Initadmin extends Core
{
	/**
	 * @var integer переменная, передаваемая в URL: номер редактируемого элемента
	 */
	public $edit;

	/**
	 * @var integer переменная, передаваемая в URL: номер сохраняемого элемента
	 */
	public $save;

	/**
	 * @var integer переменная, передаваемая в URL: сохранить новый элемент (1)
	 */
	public $savenew;

	/**
	 * @var integer переменная, передаваемая в URL: номер элемента раскрытия каталога вложенности
	 */
	public $parent;

	/**
	 * @var integer переменная, передаваемая в URL: номер системного сообщения
	 */
	public $error;

	/**
	 * @var integer переменная, передаваемая в URL: добавление элемента (1)
	 */
	public $addnew;

	/**
	 * @var integer переменная, передаваемая в URL: номер категории
	 */
	public $cat;

	/**
	 * @var integer переменная, передаваемая в URL: номер страницы
	 */
	public $site;

	/**
	 * @var integer номер страницы
	 */
	public $page;

	/**
	 * @var string ЧПУ текущей страницы
	 */
	public $rewrite;

	/**
	 * @var string модуль текущей страницы
	 */
	public $module;

	/**
	 * @var string название подключаемой функции
	 */
	public $action;

	/**
	 * @var string подключаемый класс
	 */
	public $include_class;

	/**
	 * @var integer номер ошибки
	 */
	public $err;

	/**
	 * @var array переменные, передаваемые в URL
	 */
	public $rewrite_variable_names = array(
		'edit',
		'savenew',
		'save',
		'addnew',
		'site',
		'cat',
		'parent',
		'page',
		'error'
	);

	/**
	 * @var integer номер текущей страницы
	 */
	public $cid;

	/**
	 * @var string заголовок текущей страницы
	 */
	public $name;

	/**
	 * @var string контент текущей страницы
	 */
	public $text;

	/**
	 * @var integer номер страницы-родителя
	 */
	public $parent_id;
	
	/**
	 * @var object работа с пользователями
	 */
	public $_user;

	/**
	 * @var object кэширование
	 */
	public $_cache;
	
	/**
	 * @var object ЧПУ
	 */
	public $_route;
	
	/**
	 * @var object шаблоны
	 */
	public $_tpl;
	
	/**
	 * @var array объекты модулей
	 */
	private $_objects;

	/**
	 * Конструктор класса. Определяет свойства класса
	 * 
	 * @return void
	 */
	public function __construct()
	{
		$this->_user = new User_admin();

		include_once(ABSOLUTE_PATH.'includes/view.php');

		include_once(ABSOLUTE_PATH.'includes/route.php');
		$this->_route = new Route($this);

		require_once(ABSOLUTE_PATH.'includes/template.php');
		$this->_tpl = new Template($this);
	}

	/**
	 * Подключает вспомогательные модули, если они не подключены
	 * 
	 * @return boolean true
	 */
	public function __get($name)
	{
		if (! isset($this->_objects[$name]))
		{
			// подключаем обработчик запросов модуля
			if (substr($name, 0, 1) == '_')
			{
				$module = substr($name, 1);
				if (file_exists(ABSOLUTE_PATH.'modules/'.$module.'/'.$module.'.view.php'))
				{
					include_once(ABSOLUTE_PATH.'modules/'.$module.'/'.$module.'.view.php');
				}
				if (file_exists(ABSOLUTE_PATH.'modules/'.$module.'/'.$module.'.inc.php'))
				{
					include_once(ABSOLUTE_PATH.'modules/'.$module.'/'.$module.'.inc.php');
					$class = ucfirst($module).'_inc';
					$this->_objects[$name] = new $class($this);
				}
			}
			else
			{
				return '';
			}
		}
		return $this->_objects[$name];
	}

	/**
	 * Инициализирует генерирование страницы
	 * 
	 * @return boolean true
	 */
	public function init()
	{
		Dev::set_profiling();

		Session_admin::init();

		$this->include_other_file();

		include_once(ABSOLUTE_PATH.'includes/cache.php');
		$this->_cache = new Cache;
		$this->magic_quote_off();

		define('BASE_PATH', "http://".$_SERVER["HTTP_HOST"]."/".(REVATIVE_PATH ? REVATIVE_PATH.'/' : ''));
		define('BASE_URL', $_SERVER["HTTP_HOST"].(REVATIVE_PATH ? '/'.REVATIVE_PATH : ''));

		$this->get_language_array();
		$this->get_user();

		$this->set_timezone();

		Gzip::init_gzip();

		$this->reminding();
		$this->prepare_rewrite();
		$this->prepare_variables();

		$this->get_page_in_site();

		$this->get_headers();
		$this->get_module();

		Dev::get_profiling();
		Gzip::do_gzip();
		return TRUE;
	}

	/**
	 * Подключает файлы для решения нестандартых задач (вывод каптчи...)
	 * 
	 * @return boolean true
	 */
	private function include_other_file()
	{
		if (! empty($_GET["rewrite"]))
		{
			$rewrite_array = explode("/", $_GET["rewrite"]);
			if (count($rewrite_array) > 1 && ! in_array($rewrite_array[1], array('model', 'view', 'admin', 'inc', 'ajax'))
			   && file_exists(ABSOLUTE_PATH.'modules/'.$rewrite_array[0].'/'.$rewrite_array[0].'.'.$rewrite_array[1].'.php'))
			{
				$path = ABSOLUTE_PATH.'modules/'.$rewrite_array[0].'/'.$rewrite_array[0].'.'.$rewrite_array[1].'.php';
				unset($rewrite_array[0]);
				unset($rewrite_array[1]);
				$_GET["rewrite"] = implode('/', $rewrite_array);
				include_once($path);
			}
		}
		return TRUE;
	}

	/**
	 * Записывает данные о языках сата
	 * 
	 * @return boolean true
	 */
	private function get_language_array()
	{
		unset($_lang);
		$this->language_all();
		include_once(ABSOLUTE_PATH.'language/'.LANGUAGE_BASE.'.php');

		if ($_GET["rewrite"])
		{
			$rew = explode('/', $_GET["rewrite"], 2);
			foreach ($this->language_all as $l => $langarr)
			{
				if (in_array($rew[0], $langarr))
				{
					$_GET["rewrite"] = preg_replace('/^'.$rew[0].'(\/)*/', '', $_GET["rewrite"]);
					define('_LANG', $langarr[0]);
					define('_SHORTNAME', $langarr[1].'/');
					define('TITLE', (defined('TIT'.$langarr[0]) ? constant('TIT'.$langarr[0]) : ''));
					define('BASE_PATH_HREF', "http://".$_SERVER["HTTP_HOST"]."/".(REVATIVE_PATH ? REVATIVE_PATH.'/' : '').ADMIN_FOLDER.'/'.$langarr[1].'/');
					break;
				}
			}
		}
		if (! defined('_LANG'))
		{
			define('_LANG', '');
			define('TITLE', TIT);
			define('_SHORTNAME', '');
			define('BASE_PATH_HREF', "http://".$_SERVER["HTTP_HOST"]."/".(REVATIVE_PATH ? REVATIVE_PATH.'/' : '').ADMIN_FOLDER.'/');
		}
		return TRUE;
	}

	/**
	 * Инициирует авторизацию или выход пользователя из системы
	 * 
	 * @return boolean true
	 */
	private function get_user()
	{
		if (! empty($_POST['action']) && $_POST['action'] == 'auth')
		{
			$this->_user->auth($_POST);
		}
		elseif (strstr($_GET["rewrite"], "logout"))
		{
			$this->_user->logout();
		}
		return TRUE;
	}

	/**
	 * Выводит форму восстановления пароля
	 * 
	 * @return boolean true
	 */
	private function reminding()
	{
		if (empty($this->_user->id) && ! empty($_GET["rewrite"]) && ($_GET["rewrite"] == "reminding" || $_GET["rewrite"] == "reminding/diafan"))
		{
			$this->rewrite = $_GET["rewrite"];
			$this->module = "reminding";
			include_once(ABSOLUTE_PATH.'adm/includes/theme.php');
			include_once(ABSOLUTE_PATH.'includes/model.php');
			include_once(ABSOLUTE_PATH.'adm/includes/reminding.php');
			include_once(ABSOLUTE_PATH.'modules/reminding/reminding.ajax.php');
			include_once(ABSOLUTE_PATH.'modules/reminding/reminding.view.php');
			$reminding = new Reminding_admin($this);
			$reminding->form();
			$theme = new Theme_admin($this);
			include_once(ABSOLUTE_PATH.'adm/themes/adminreminding.php');
			exit;
		}
		return TRUE;
	}

	/**
	 * Подготавливает запрос для идентифицикации страницы в таблице {site} по rewrite или по id,
	 * удаляет из строки запроса $_GET[rewrite] переданные переменные
	 * 
	 * @return boolean true
	 */
	private function prepare_rewrite()
	{
		if ($_GET["rewrite"])
		{		
			$rewrite_array = explode("/", $_GET["rewrite"]);
			
			foreach ($rewrite_array as $key => $ra)
			{
		
				foreach ($this->rewrite_variable_names as $name)
				{		
					if (preg_match('/'.$name.'([0-9]+)/', $ra, $result))
					{
						$this->$name = $result[1];
						unset($rewrite_array[$key]);
					}
				}
			}
			$this->rewrite = implode("/", $rewrite_array);
		}
		if (! $this->rewrite)
		{
			if (! $this->admin_roles('init', 'site') || ! $this->_user->id)
			{
				$this->rewrite = 'site';
			}
			else
			{
				$result = DB::query("SELECT id, rewrite FROM {adminsite} WHERE act='1' ORDER BY id ASC"); 
				while ($row = DB::fetch_array($result))
				{
					if (! $this->admin_roles('init', $row["rewrite"]))
					{
						$this->rewrite = $row["rewrite"];
						break;
					}
				}
			}
		}
		return TRUE;
	}

	/**
	 * Подключает файл, выполняющий запрашиваемое действие на элементом
	 * 
	 * @return boolean true
	 */
	private function prepare_variables()
	{
		define('URL', path_module($this));

		include(ABSOLUTE_PATH."adm/includes/theme.php");
		
		if (! empty($_POST["action"]))
		{
			// подключаем обработчик запросов модуля
			if (! empty($_POST["module"]))
			{
				$module = preg_replace('/[^a-z0-9_]+/', '', $_POST["module"]);
				if (file_exists(ABSOLUTE_PATH.'modules/'.$module.'/admin/'.$module.'.admin.ajax.php'))
				{
					include_once(ABSOLUTE_PATH."adm/includes/ajax.php");

					include_once(ABSOLUTE_PATH.'modules/'.$module.'/admin/'.$module.'.admin.ajax.php');
					$this->include_class = ucfirst($module).'_admin_ajax';
					$this->action = 'ajax_module';
					return TRUE;
				}
			}
			switch($_POST["action"])
			{
				case 'trash':
				case 'delete':
					include(ABSOLUTE_PATH."adm/includes/del.php");
					$this->action = 'del_module';
					return TRUE;

				case 'restore':
					$this->action = 'restore_module';
					include(ABSOLUTE_PATH."adm/includes/del.php");
					return TRUE;

				case 'up':
				case 'down':
					include(ABSOLUTE_PATH."adm/includes/order.php");
					$this->action = 'order_module';
					return TRUE;

				case 'unblock':
				case 'block':
					include(ABSOLUTE_PATH."adm/includes/act.php");
					$this->action = 'act_module';
					return TRUE;

				case 'sort':
					include(ABSOLUTE_PATH."adm/includes/ajax.php");
					$this->action = 'sort_module';
					return TRUE;

				case 'parent':
					include(ABSOLUTE_PATH."adm/includes/ajax.php");
					$this->action = 'parent_id_module';
					return TRUE;
			}
		}

		if ($this->save)
		{
			include(ABSOLUTE_PATH."adm/includes/save.php");
			$this->action = 'save_module';
			return TRUE;
		}

		if ($this->savenew && ! empty($_POST["id"]))
		{
			$this->save = intval($_POST["id"]);
			include(ABSOLUTE_PATH."adm/includes/save.php");
			$this->action = 'save_module';
			return TRUE;
		}
		
		if ($this->savenew)
		{
			include(ABSOLUTE_PATH."adm/includes/save.php");
			$this->action = 'new_module';
			return TRUE;
		}

		if ($this->edit || $this->addnew || strstr($this->rewrite, "config"))
		{
			if (! $this->edit)
			{
				$this->edit = 1;
			}
			include(ABSOLUTE_PATH."adm/includes/edit.php");
		}

		else
		{
			include_once(ABSOLUTE_PATH."adm/includes/show.php");
		}

		$this->action = 'show_theme';
		return TRUE;
	}

	/**
	 * Идентифицирует страницу, задает параметры страницы
	 * 
	 * @return boolean true
	 */
	private function get_page_in_site()
	{
		if ($this->rewrite)
		{
			$row = DB::fetch_array(DB::query("SELECT id, name, text, parent_id FROM {adminsite} WHERE rewrite = '%h'", $this->rewrite));
		}

		if (empty($row))
		{
			include(ABSOLUTE_PATH.'includes/404.php');
		}

		$this->cid       = $row["id"];
		$this->name      = $row['name'];
		$this->text      = $row['text'];
		$this->parent_id = $row['parent_id'];

		return TRUE;
	}

	/**
	 * Отдает браузеру заголовки страницы
	 * 
	 * @return boolean true
	 */
	private function get_headers()
	{
		header("Expires: " . date("r"));
		header("Cache-Control: no-store, no-cache, must-revalidate");
		header("Pragma: no-cache");
		header("Cache-Control: post-check=0, pre-check=0", FALSE);
		header('Content-Type: text/html; charset=utf-8');

		return TRUE;
	}

	/**
	 * Подключает модуль
	 * 
	 * @return boolean true
	 */
	private function get_module()
	{
		if ($this->admin_roles('init') && $this->_user->id)
		{
			include(ABSOLUTE_PATH.'includes/404.php');
		}
		if (strstr($this->rewrite, '/'))
		{
			$module = preg_replace('/[^A-Za-z0-9-\.]+/', '', str_replace('/', '.admin.', $this->rewrite));
			$rew = explode('/', $this->rewrite);
			$rewrite = preg_replace('/[^A-Za-z0-9-]+/', '', $rew[0]);
		}
		else
		{
			$rewrite = preg_replace('/[^A-Za-z0-9-]+/', '',$this->rewrite);
			$module = $rewrite.'.admin';
		}

		if (file_exists(ABSOLUTE_PATH.'modules/'.$rewrite.'/admin/'.$module.'.php') && $this->_user->id)
		{
			if (! empty($_GET["parent"]))
			{
				$this->parent = preg_replace('/[^0-9]+/', '', $_GET["parent"]);
			}

			include_once(ABSOLUTE_PATH.'modules/'.$rewrite.'/admin/'.$module.'.php');
		}
		$this->module = $rewrite;
		$name_class_module = str_replace('.', '_', ucfirst($module));
		$name_func_module = 'inc_file_'.$rewrite;

		if (function_exists($name_func_module))
		{
			$name_class_module = $name_func_module($this);
		}

		if (in_array($name_class_module, get_declared_classes()))
		{
			$frame = new $name_class_module($this);
			$this->get_config_module($frame);
		}

		else
		{
			$frame = new Frame_admin($this);
		}

		if ($this->parent && $frame->config_module["parent"] && ! DB::query_result("SELECT id FROM {".$frame->table."} WHERE id=%d", $this->parent))
		{
			$this->redirect(path_module($this, 0, 1));
			exit;
		}

		if ($this->parent && ! $frame->config_module["parent"] && ! DB::query_result("SELECT id FROM {".$frame->table."_category} WHERE id=%d", $this->parent))
		{
			$this->redirect(path_module($this, 0, 1));
			exit;
		}

		if ($this->site && $frame->config_module["element_site"] && ! DB::query_result("SELECT id FROM {site} WHERE id=%d", $this->site))
		{
			$this->redirect(BASE_PATH_HREF.$this->rewrite.'/');
			exit;
		}
		if ($this->action)
		{
			$action = $this->action;
			$frame->$action();
		}

		return TRUE;
	}

	/**
	 * Определяет права доступа
	 * 
	 * @param string $module_name модуль
	 * @param string $action действие
	 * @param array $roles права пользователя
	 * @return boolean
	 */
	public function admin_roles($action, $module_name = '', $roles = '')
	{
		if (! $module_name)
		{
			$module_name = $this->rewrite;
		}
		if (! $roles)
		{
			$roles = $this->_user->roles;
		}
		return (! in_array('admin all', $roles) && ! in_array('admin '.$module_name.' all', $roles) && ! in_array('admin '.$module_name.' '.$action, $roles));
	}

	/**
	 * Задает настройку модуля по умолчанию
	 * 
	 * @return boolean true
	 */
	public function get_config_module(&$th)
	{
		$config_module = array('act', 'del', 'nopage', 'element', 'element_multiple', 'element_site', 'category', 'category_flat', 'category_rel', 'date', 'datetime',
		                       'parent', 'menu', 'view', 'order', 'trash', 'no_out', 'tab_card', 'search_name', 'config', 'link_to_element', 'image', 'prior');

		foreach ($config_module as $param)
		{
			if (empty($th->config_module[$param]))
			{
				$th->config_module[$param] = '';
			}
		}

		return TRUE;
	}

	/**
	 * Борьба с магическими кавычками
	 * 
	 * @return boolean true
	 */
	private function magic_quote_off()
	{
		if (get_magic_quotes_gpc())
		{
			$_GET  = $this->stripslashes_array($_GET);
			$_POST = $this->stripslashes_array($_POST);
		}
		return TRUE;
	}

	/**
	 * Вырезает магические кавычки из массива
	 * 
	 * @return array
	 */
	private function stripslashes_array($array)
	{
		if (is_array($array))
		{
			foreach ($array as $key => $value)
			{
				$array[$key] = $this->stripslashes_array($value);
			}
			return $array;
		}
		else
		{
			return stripslashes($array);
		}
	}

	/**
	 * Устанавливает часовой пояс
	 * 
	 * @return boolean true
	 */
	private function set_timezone()
	{
		if (! TIMEZONE || @ ! date_default_timezone_set(TIMEZONE)) 
		{
			date_default_timezone_set('Europe/Moscow');
		}
		return TRUE;
	}
}