<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.4
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include(dirname(dirname(dirname(__FILE__))).'/includes/404.php');
}

/**
 * Comments_ajax
 *
 * Обработка запроса при добавления комментария
 */
class Comments_ajax extends Model
{
	/**
	 * @var object основной объект системы
	 */
	public $diafan;

	/**
	 * @var array внутренние сообщения
	 */
	private $result;

	/**
	 * Конструктор класса
	 * 
	 * @return void
	 */
	public function __construct(&$diafan)
	{
		$this->diafan = &$diafan;
	}

	/**
	 * Обрабатывает полученные данные из формы
	 * 
	 * @return boolean
	 */
	public function ajax_request()
	{
		if (empty($_POST['module']) || $_POST['module'] != 'comments')
		{
			return FALSE;
		}
		if (! empty($_POST["action"]) && $_POST["action"] == "list")
		{
			$this->diafan->_comments->get();
			return TRUE;
		}
		if (! $this->diafan->configmodules('security_user', 'comments') || $this->diafan->_user->id)
		{
			$id = $this->diafan->module
			      ? ($this->diafan->show ? $this->diafan->show : $this->diafan->cat)
			      : $this->diafan->cid;

			$module_name = $this->diafan->module
			               ? $this->diafan->module.(! $this->diafan->show ? "_category" : '')
			               : 'site';

			$this->check_captcha();
			$this->valid_text();

			if ($this->send_errors())
				return TRUE;

			if ($this->error_insert($id, $module_name))
				return TRUE;

			if (! DB::query("INSERT INTO {comments} (created, module_name,element_id, user_id, text,act) VALUES ('%d', '%h','%d','%d','%h','%d')",
			               time(),
			               $module_name,
			               $id,
			               $this->diafan->_user->id,
			               $_POST["comment"],
			               $this->diafan->configmodules('security_moderation', 'comments') ? 0 : 1
			              )
			  )
			{
				$this->result["errors"][0] = $this->diafan->configmodules('error_add_message', 'comments');
				return $this->send_errors();
			}
			//модерация сообщений
			if ($this->diafan->configmodules('security_moderation', 'comments'))
			{
				$mes = $this->diafan->configmodules('add_message', 'comments');
				$this->result["errors"][0] = $mes ? $mes : ' ';
				return $this->send_errors(TRUE);
			}

			if (! empty($_SERVER["HTTP_X_REQUESTED_WITH"]))
			{
				//выводит добавленное сообщение
				$row['date'] = $this->format_date(time(), "comments");
				$row["text"] = nl2br($this->diafan->get_param($_POST, "comment", '', 1));
				if ($this->diafan->configmodules('user_name', 'comments') && $this->diafan->_user->id)
				{
					$row["name"] = $this->get_author($this->diafan->_user->id);
				}
				$result["rows"][] = $row;
				$result["new"]    = !empty($_POST["first"]) ? FALSE : TRUE;

				$this->result["add"] = $this->diafan->_tpl->get('get', 'comments', $result);
			}
			else
			{
				$mes = $this->diafan->configmodules('add_message', 'comments');
				$this->result["errors"][0] = $mes ? $mes : ' ';
			}
			$this->result["success"] = TRUE;

			return $this->send_errors();
		}
	}

	/**
	 * Проверяет правильность каптчи
	 * 
	 * @return boolean
	 */
	private function check_captcha()
	{
		$this->result["code"] = '';
		if ($this->diafan->configmodules('security_code', 'comments') == 1)
		{
			$mes = $this->diafan->_captcha->is_right('comments');
			$this->result["code"] = $this->diafan->_captcha->get('comments', '', TRUE);	
			if ($mes)
			{
				$this->result["errors"]["code"] = $mes;
				return TRUE;
			}
		}
		return FALSE;
	}

	/**
	 * Проверяет валидность комментария
	 * 
	 * @return boolean true
	 */
	private function valid_text()
	{
		if (! $_POST["comment"])
		{
			$mes = _LANG_EMPTY_COMMENT;
		}
		else
		{
			require_once(ABSOLUTE_PATH.'includes/validate.php');
			$mes = Validate::text($_POST["comment"]);
		}
		if ($mes)
		{
			$this->result["errors"][0] = $mes;
		}

		return TRUE;
	}

	/**
	 * Проверяет попытку отправить сообщение повторно
	 *
	 * @param integer $element_id номер элемента, к которому добавляется комментарий
	 * @param string $module_name модуль, к которому добавляется комментарий
	 * @return boolean
	 */
	private function error_insert($element_id, $module_name)
	{
		$mes = '';
		$num = DB::query_result("SELECT COUNT(id) FROM {comments} WHERE element_id='%d' AND module_name='%h' AND user_id='%d' AND text='%h' AND trash='0'",
		                        $element_id,
		                        $module_name,
		                        $this->diafan->_user->id,
		                        $_POST["comment"]
		                       );

		if ($num > 0)
		{
			$mes = $this->diafan->configmodules('error_insert_message', 'comments');
			$this->result["errors"][0] = $mes ? $mes : ' ';
		}
		return $this->send_errors();
	}

	/**
	 * Отправляет несколько ошибок
	 *
	 * @return boolean
	 */
	private function send_errors()
	{
		if (! empty($this->result["errors"]) || ! empty($this->result["success"]))
		{
			if (! empty($_POST['ajax']))
			{
				include_once(ABSOLUTE_PATH.'plugins/json.php');
				echo to_json($this->result);
			}
			else
			{
				$query = '';
				if (! empty($this->result["errors"]))
				{
					foreach ($this->result["errors"] as $field => $mes)
					{
						if (empty($_POST["update"]))
						{
							$query .= '&mess'.($field ? '-'.$field : '').'='.$mes;
						}
					}
				}
				$this->diafan->redirect($this->diafan->_route->current_link().($query ? '?error=comments'.$query : ''));
			}
			return TRUE;
		}
		return FALSE;
	}
}