<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.4
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include(dirname(dirname(dirname(__FILE__))).'/includes/404.php');
}

/**
 * Faq_admin
 *
 * Редактирование вопросов
 */
class Faq_admin extends Frame_admin
{
	/**
	 * @var string название файла. используется в имени констант, определяющих названия полей
	 */
	public $file = 'faq';

	/**
	 * @var string таблица в базе данных
	 */
	public $table = 'faq'; 

	/**
	 * Конструктор класса
	 * 
	 * @return void
	 */
	public function __construct($diafan)
	{
		$this->diafan = $diafan;

		//поля в базе данных для редактирования
		$this->variable_table = array(
			'name'        => 'text',
			'created'     => 'datetime',
			'act'         => 'checkbox',
			'often'       => 'checkbox',
			'sendmail'    => 'function',
			'mail'        => 'text',
			'attachments' => 'module',
			'tags'        => 'module',
			'comments'    => 'module',
			'rating'      => 'module',
			'cat_id'      => 'function',
			'hr1'         => 'hr',
			'menu'        => 'module',
			'number'      => 'function',
			'timeedit'    => 'function',
			'title_meta'  => 'text',
			'keywords'    => 'text',
			'descr'       => 'textarea',
			'rewrite'     => 'function',
			'site_id'     => 'function',
			'map_no_show' => 'checkbox',
			'hr2'         => 'hr',
			'anons'       => 'textarea',
			'text'        => 'editor'
		);
		
		//поля, которые прячутся в Дополнительно
		$this->variable_other = array(
			'keywords'    => 1,
			'title_meta'  => 1,
			'descr'       => 1,
			'site_id'     => 1,
			'rewrite'     => 1,
			'timeedit'    => 1,
			'number'      => 1,
			'hr1'         => 1,
			'map_no_show' => 1,
			'menu'        => 1
		);

		//переводимые поля
		$this->language_names = array(
			'name',
			'title_meta',
			'anons',
			'act',
			'keywords',
			'descr',
			'text'
		);

		$this->config_module = array(
			'act'              => 1, //показать/скрыть
			'menu'             => 1, //используется в меню
			'del'              => 1, //удалить
			'date'             => 1, //показывать дату в списке, сортировать по дате
			'element_site'     => 1, //модуль может быть прикреплен к нескольким страницам
			'element'          => $this->diafan->configmodules("cat", "faq", $this->diafan->site) ? 1 : 0, //используются группы
			'element_multiple' => $this->diafan->configmodules("cat", "faq", $this->diafan->site) ? 1 : 0, //модуль может быть прикреплен к нескольким группам
			'trash'            => 1  //использовать корзину
		);

		//выводить в списке содержание полей:
		$this->config_other_row = array(
			'anons',
			'text'
		);

		//высота поля для Визуального редактора
		$this->editor_height = array(
			'anons' => 200,
			'text'  => 400
		);
	}

	/**
	 * Выводит список вопросов
	 * @return boolean true
	 */
	public function show_module()
	{
		if ($this->config_module['element'] && ! $this->not_empty_categories)
		{
			echo '<p class="red">'.sprintf(_ADMIN_FAQ_EMPTY_CATEGORIES, '<a href="'.BASE_PATH_HREF.'faq/category/'.($this->diafan->site ? 'site'.$this->diafan->site.'/' : '').'">', '</a>').'</p>';
		}
		else
		{
			$this->addnew_init(_ADMIN_FAQ_ADD);
		}
		echo '<table class="list">';
			$this->list_row();
		echo '</table>';

		return TRUE;
	}

	/**
	 * Выводит статус вопроса (отвеченный/неотвеченный), в списке
	 * 
	 * @return string
	 */
	protected function other_row_anons($row)
	{
		return (! $row["text"] ? ' <a href="'.path_module($this->diafan).'edit'.$row["id"].'/">(неотвеченный вопрос)</a>' : '').'</td><td class="comment">
			<div>
				<span></span>
				<table><tbody><tr><td>
					'.$this->diafan->short_text($row["anons"]).'
				</td></tr></tbody></table>
			</div>';
	}

	/**
	 * Редактирование поля "Отправить ответ"
	 * @return boolean true
	 */
	protected function edit_sendmail_module()
	{
		echo '
		<tr>
			<td class="td_first">'.$this->variable_names().'</td>
			<td>
				<input name="sendmail" value="1" type="checkbox">'.$this->show_help_row().'
			</td>
		</tr>';
	}

	/**
	 * Сохранение поля "Отправить ответ"
	 * @return boolean true
	 */
	protected function save_sendmail_module()
	{		
		if (! empty($_POST["sendmail"]) && ! empty($_POST["mail"]) && ! empty($_POST["text"]) && ! empty($_POST["anons"]))
		{
			$subject = str_replace(
				array('%title', '%url'),
				array(TITLE, BASE_URL),
				$this->diafan->configmodules('subject', 'faq', $_POST["site_id"])
			);

			$message = str_replace(
				array(
					', %name',
					'%name',
					'%title',
					'%url',
					'%question',
					'%answer'
				),
				array(
					strip_tags($_POST["name"]) ? ', '.strip_tags($_POST["name"]) : '',
					strip_tags($_POST["name"]),
					TITLE,
					BASE_URL,
					htmlspecialchars($_POST["anons"]),
					$_POST["text"]
				),
				$this->diafan->configmodules('message', 'faq', $_POST["site_id"])
			);

			include_once(ABSOLUTE_PATH.'includes/mail.php');
			send_mail(
				trim(strip_tags($_POST["mail"])),
				$subject,
				$message,
				$this->diafan->configmodules("emailconf", 'faq', $_POST["site_id"]) && $this->diafan->configmodules("email", 'faq', $_POST["site_id"])
				? $this->diafan->configmodules("email",'faq',$_POST["site_id"])
				: ''
			);
			$this->diafan->err = 5;
		}
		elseif (! empty($_POST["sendmail"]))
		{
			$this->diafan->err = 6;
		}
		return TRUE;
	}
}