<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.4
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include(dirname(dirname(dirname(__FILE__))).'/includes/404.php');
}

/**
 * Feedback_model
 *
 * Модель модуля "Обратная связь"
 */
class Feedback_model extends Model
{
	/**
	 * @var array сгенерированные в моделе данные, передаваемые в шаблон
	 */
	protected $result;

	/**
	 * @var object основной объект системы
	 */
	protected $diafan;

	/**
	 * Конструктор класса
	 * 
	 * @return void
	 */
	public function __construct($diafan)
	{
		$this->diafan = $diafan;
	}

	/**
	 * Генерирует данные для формы добавления сообщения
	 * 
	 * @param integer $site_id номер страницы
	 * @param boolean $insert_form форма выводится с помощью шаблонного тэга
	 * @return array|boolean false
	 */
	public function form($site_id = 0, $insert_form = FALSE)
	{
		if (! $insert_form)
		{
			$site_id = $this->diafan->cid;
		}
		else
		{
			if (! $site_id)
			{
				$site_id = DB::query_result("SELECT id FROM {site} WHERE addmodule='feedback' LIMIT 1");
			}
		}
		if (! $site_id)
		{
			return FALSE;
		}
		$this->result["site_id"] = $site_id;

		if ($this->diafan->configmodules('security', "feedback", $site_id) != 1 || $this->diafan->_user->id)
		{
			$this->result["attachments"] = 0;
			$this->result["captcha"] = '';
			if ($this->diafan->configmodules('security', "feedback", $site_id) == 2)
			{
				$this->result["captcha"] = $this->diafan->_captcha->get("feedback", $this->get_error("feedback", "code"));
			}

			if ($this->diafan->configmodules('attachments', "feedback", $site_id) && $this->diafan->configmodules("max_count_attachments", "feedback", $site_id))
			{
				$this->result["attachments"]           = TRUE;
				$this->result["max_count_attachments"] = $this->diafan->configmodules("max_count_attachments", "feedback", $site_id);
				$this->result["error_attachments"]     = $this->get_error("feedback", 'attachments');
			}
		}
		else
		{
			$this->result["text"] = _LANG_NECESSARY_REGISTRATION;
		}

		$cache_meta = array(
				    "name"     => "form_param",
				    "language" => _LANG,
				    "site_id"  => $site_id
				    );
		if (! $rows_param = $this->diafan->_cache->get($cache_meta, "feedback"))
		{
			$rows_param = array();
			$result = DB::query("SELECT id, [name], type, required, [text] FROM {feedback_param} WHERE site_id=%d"
					    ." AND trash='0' ORDER BY sort ASC", $site_id);

			while ($row = DB::fetch_array($result))
			{
				if ($row["type"] == 'select' || $row["type"] == 'multiple')
				{
					$result_select = DB::query("SELECT [name], id FROM {feedback_param_select} WHERE param_id=%d"
								   ." ORDER BY sort ASC", $row["id"]);
					while ($row_select = DB::fetch_array($result_select))
					{
						$row["select_array"][] = $row_select;
					}
				}
				$rows_param[] = $row;
			}
			//сохранение кеша
			$this->diafan->_cache->save($rows_param, $cache_meta, "feedback");
		}
		foreach ($rows_param as $row)
		{
			$this->result['error_p'.$row["id"]] = $this->get_error("feedback", 'p'.$row["id"]);
		}

		$this->result["error"]  = $this->get_error("feedback");
		$this->result["rows"]   = $rows_param;
		return $this->result;
	}
}