<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.4
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include(dirname(dirname(dirname(__FILE__))).'/includes/404.php');
}

/**
 * Images_admin_ajax
 * 
 * Обработка Ajax-запросов при работе с изображениями в административной части
 */
class Images_admin_ajax
{
	/**
	 * @var object основной объект системы
	 */
	public $diafan;

	/**
	 * @var array настройки отображения
	 */
	public $config_module;

	/**
	 * @var string таблица в базе данных
	 */
	public $table;

	/**
	 * @var array результаты, передаваемы Ajaxом
	 */
	private $result;

	/**
	 * Конструктор класса
	 * 
	 * @return void
	 */
	public function __construct($frame)
	{
		$this->diafan = $frame->diafan;
		$this->config_module = $frame->config_module;
		$this->table  = $frame->table;

		// Прошел ли пользователь проверку идентификационного хэша
		if (! $this->diafan->_user->checked)
		{
			$this->result["error"] = 'ERROR';
			$this->send_json();
		}

		$this->result["hash"] = $this->diafan->_user->get_hash();

		if (! empty($_POST["action"]))
		{
			switch($_POST["action"])
			{
				case 'upload':
					$this->upload();
					break;

				case 'delete':
					$this->delete();
					break;

				case 'edit':
					$this->edit();
					break;

				case 'save':
					$this->save();
					break;

				case 'up':
				case 'down':
					$this->sort();
					break;

				case 'upload_watermark':
					$this->upload_watermark();
					break;

				case 'delete_watermark':
					$this->delete_watermark();
					break;

				case 'change_watermark':
					$this->change_watermark();
					break;

				case 'resize':
					$this->resize();
					break;
			}
		}
	}

	/**
	 * Загружает изображение
	 * 
	 * @return void
	 */
	private function upload()
	{
		if ($this->diafan->admin_roles('save'))
		{
			$this->result["error"] = _ADMIN_NO_RULES_FOR_SAVE;
			$this->send_json();
		}

		if (! isset($_FILES["image"]) || ! is_array($_FILES["image"]) || $_FILES["image"]['name'] == '')
		{
			$this->result["errors"]["image"] = _ADMIN_NO_FILE_FOR_UPLOAD;
			$this->send_json();
		}

		include_once(ABSOLUTE_PATH."includes/image.php");

		$this->result["id"] = $this->get_id_element();

		$new_name = strtolower($this->diafan->translit($_FILES["image"]['name']));
		$extension = substr(strrchr($new_name, '.'), 1);
		$new_name = substr($new_name, 0, - (strlen($extension) + 1));
		if (strlen($new_name) + strlen($extension) > 49)
		{
			$new_name = substr($new_name, 0, 49 - strlen($extension));
		}
		$mimes = array(
			'gif' => 'image/gif',
			'jpeg' => 'image/jpeg',
			'jpg' => 'image/jpeg',
			'jpe' => 'image/jpeg',
			'png' => 'image/png'
		);

		if (! in_array($extension, array_keys($mimes)) || empty($mimes[$extension]) || $mimes[$extension] != strtolower($_FILES["image"]['type']))
		{
			return '<br>'.sprintf(_LANG_ATTACHMENT_EXTENSIONS, 'gif, jpeg, jpg, jpe, png');
		}

		$config_cat = $this->config_module['category'] ? "_cat" : "";

		if (! $this->diafan->configmodules("big_width".$config_cat) || ! $this->diafan->configmodules("big_height".$config_cat))
		{
			$this->result["errors"]["image"] = _ADMIN_NO_CONFIG_BIG_SIZE;
			$this->send_json();
		}

		if (! $this->diafan->configmodules("medium_width".$config_cat) || ! $this->diafan->configmodules("medium_height".$config_cat))
		{
			$this->result["errors"]["image"] = _ADMIN_NO_CONFIG_MEDIUM_SIZE;
			$this->send_json();
		}
		if ($this->diafan->configmodules("watermark"))
		{
			if (! $this->diafan->configmodules("watermark_path"))
			{
				$this->result["errors"]["image"] = _ADMIN_NO_CONFIG_WATERMARK;
				$this->send_json();
			}
		}

		list($width, $height) = getimagesize($_FILES["image"]['tmp_name']);
		if ($width > $this->diafan->configmodules("big_width".$config_cat) || $height > $this->diafan->configmodules("big_height".$config_cat))
		{
			if (! Image::resize($_FILES["image"]['tmp_name'],
				       $this->diafan->configmodules("big_width".$config_cat),
				       $this->diafan->configmodules("big_height".$config_cat),
				       $this->diafan->configmodules("big_quality".$config_cat)
			))
			{
				$this->result["errors"]["image"] = _ADMIN_INCORRECT_FORMAT_FILE;
				$this->send_json();
			}
		}

		$size = filesize($_FILES["image"]['tmp_name']);

		$new_name = $new_name.'.'.$extension;

		DB::query("INSERT INTO {images} (name, module_name, element_id, size) VALUES ('%s', '%s', '%d', '%d')",
		          $new_name,
		          $this->diafan->module.($this->config_module['category'] ? "cat" : ""),
		          $this->result["id"],
		          $size
		);

		$id = DB::next_id("images");
		$new_name = $id.'_'.$new_name;
		DB::query("UPDATE {images} SET name='%h' WHERE id=%d", $new_name, $id);
		if (! copy($_FILES["image"]['tmp_name'], ABSOLUTE_PATH.USERFILES."/".$this->diafan->module."/".$new_name))
		{
			$this->result["errors"]["image"] = _ADMIN_NO_UPLOAD_FILE;
			$this->send_json();
		}

		if ($this->diafan->configmodules("watermark"))
		{
			Image::watermark(ABSOLUTE_PATH.USERFILES."/".$this->diafan->module."/".$new_name,
			                 ABSOLUTE_PATH.USERFILES."/watermark/".$this->diafan->configmodules("watermark_path"),
			                 $this->diafan->configmodules("watermark_placement"));
		}

		list($width, $height) = getimagesize($_FILES["image"]['tmp_name']);

		if ($width > $this->diafan->configmodules("medium_width".$config_cat) || $height > $this->diafan->configmodules("medium_height".$config_cat))
		{
			if (! Image::resize($_FILES["image"]['tmp_name'],
					$this->diafan->configmodules("medium_width".$config_cat),
					$this->diafan->configmodules("medium_height".$config_cat),
					$this->diafan->configmodules("medium_quality".$config_cat)
			))
			{
				$this->result["errors"]["image"] = _ADMIN_INCORRECT_FORMAT_FILE;
				$this->send_json();
			}
		}

		if (! copy($_FILES["image"]['tmp_name'], ABSOLUTE_PATH.USERFILES."/".$this->diafan->module."/medium/".$new_name))
		{
			$this->result["errors"]["image"] = _ADMIN_NO_UPLOAD_FILE;
			$this->send_json();
		}
		if ($this->diafan->configmodules("watermark") && ! $this->diafan->configmodules("watermark_only_big"))
		{
			Image::watermark(ABSOLUTE_PATH.USERFILES."/".$this->diafan->module."/medium/".$new_name,
			                 ABSOLUTE_PATH.USERFILES."/watermark/".$this->diafan->configmodules("watermark_path"),
			                 $this->diafan->configmodules("watermark_placement"));
		}

		list($width, $height) = getimagesize( $_FILES["image"]['tmp_name']);

		if ($width > 50|| $height > 50)
		{		
			if (! Image::resize($_FILES["image"]['tmp_name'], 50, 50, 70))
			{
				$this->result["errors"]["image"] = _ADMIN_INCORRECT_FORMAT_FILE;
				$this->send_json();
			}
		}

		if (! move_uploaded_file($_FILES["image"]['tmp_name'], ABSOLUTE_PATH.USERFILES."/".$this->diafan->module."/small/".$new_name))
		{
			$this->result["errors"]["image"] = _ADMIN_NO_UPLOAD_FILE;
			$this->send_json();
		}

		chmod(ABSOLUTE_PATH.USERFILES."/".$this->diafan->module."/".$new_name, 0755);
		chmod(ABSOLUTE_PATH.USERFILES."/".$this->diafan->module."/medium/".$new_name, 0755);
		chmod(ABSOLUTE_PATH.USERFILES."/".$this->diafan->module."/small/".$new_name, 0755);

		$this->diafan->_cache->delete("", $this->diafan->module);

		DB::query("UPDATE {images} SET sort=id WHERE sort=0");

		include_once(ABSOLUTE_PATH.'modules/images/admin/images.admin.view.php');

		$this->result["data"]   = Images_admin_view::show($this, $this->result["id"]);
		$this->result["target"] = ".images_container";

		$this->send_json();
	}

	/**
	 * Удаляет изображение
	 * 
	 * @return void
	 */
	private function delete()
	{
		if ($this->diafan->admin_roles('del'))
		{
			$this->result["error"] = _ADMIN_NO_RULES_FOR_DELETE;
			$this->send_json();
		}
		if (! empty($_POST["element_id"]))
		{
			$element_id = intval($_POST["element_id"]);
		}
		else
		{
			$this->result["error"] = _ADMIN_NO_RULES_FOR_DELETE;
			$this->send_json();
		}

		$image_id = intval($_POST['image_id']);
		if (! empty($image_id))
		{
			$row = DB::fetch_array(DB::query("SELECT id, module_name, element_id FROM {images} WHERE id=%d and module_name='%s'", $image_id, $this->diafan->module.($this->config_module['category'] ? "cat" : "")));
			if ($row)
			{
				$this->diafan->_images->delete($row["element_id"], $row["module_name"], $row["id"]);
				$this->diafan->_cache->delete("", $this->diafan->module);
			}
		}
		$this->diafan->_cache->delete("", $this->diafan->module);

		include_once(ABSOLUTE_PATH.'modules/images/admin/images.admin.view.php');

		$this->result["data"]   = Images_admin_view::show($this, $element_id);
		$this->result["target"] = ".images_container";

		$this->send_json();
	}

	/**
	 * Редактирует изображение
	 * 
	 * @return boolean true
	 */
	private function edit()
	{
		if (! empty($_POST["element_id"]))
		{
			$element_id = intval($_POST["element_id"]);
		}
		else
		{
			$this->result["error"] = 'ERROR';
			$this->send_json();
		}

		$image_id = intval($_POST['image_id']);
		$row = DB::fetch_array(DB::query("SELECT id, [alt], [title], name FROM {images} WHERE id=%d AND element_id=%d", $image_id, $element_id));
		if (! $row)
		{
			$this->result["error"] = 'ERROR';
			$this->send_json();
		}

		$this->diafan->_cache->delete("", $this->diafan->module);

		include_once(ABSOLUTE_PATH.'modules/images/admin/images.admin.view.php');

		$this->result["data"] = Images_admin_view::edit_attribute($this, $row);
		$this->result["target"] = ".images_actions[image_id=".$image_id."]";

		$this->send_json();
		return TRUE;
	}

	/**
	 * Сохраняет данные об изображении
	 * 
	 * @return void
	 */
	private function save()
	{
		if (! empty($_POST["element_id"]))
		{
			$element_id = intval($_POST["element_id"]);
		}
		else
		{
			$this->result["error"] = 'ERROR';
			$this->send_json();
		}

		$image_id = intval($_POST['image_id']);
		DB::query("UPDATE {images} SET [alt]='%h', [title]='%h' WHERE id=%d AND element_id='%d'", $_POST["alt"], $_POST["title"], $image_id, $element_id);

		$this->diafan->_cache->delete("", $this->diafan->module);

		include_once(ABSOLUTE_PATH.'modules/images/admin/images.admin.view.php');

		$this->result["data"]   = Images_admin_view::show($this, $element_id);
		$this->result["target"] = ".images_container";

		$this->send_json();
	}

	/**
	 * Сортирует изображения
	 * 
	 * @return void
	 */
	private function sort()
	{
		if (! empty($_POST["element_id"]))
		{
			$element_id = intval($_POST["element_id"]);
		}
		else
		{
			$this->result["error"] = 'ERROR';
			$this->send_json();
		}

		$image_id = intval($_POST['image_id']);
		if (! empty($image_id))
		{
			$oldsort = DB::query_result("SELECT sort FROM {images} WHERE id=%d AND element_id=%d", $image_id, $element_id);
			$row = DB::fetch_array(DB::query("SELECT id, sort FROM {images} WHERE element_id=%d AND module_name='%s' AND id<>%d AND"
				  .($_POST["action"] == "up" ? " sort<=%d ORDER BY sort DESC" : " sort>=%d ORDER BY sort ASC")." LIMIT 1",
				  $element_id, $this->diafan->module.($this->config_module['category'] ? "cat" : ""), $image_id, $oldsort));
			if ($row)
			{
				DB::query("UPDATE {images} SET sort=%d WHERE id=%d", $oldsort, $row["id"]);
				DB::query("UPDATE {images} SET sort=%d WHERE id=%d", $row["sort"], $image_id);
			}
		}
		$this->diafan->_cache->delete("", $this->diafan->module);

		include_once(ABSOLUTE_PATH.'modules/images/admin/images.admin.view.php');

		$this->result["data"]   = Images_admin_view::show($this, $element_id);
		$this->result["target"] = ".images_container";

		$this->send_json();
	}

	/**
	 * Загружает водяной знак
	 * 
	 * @return void
	 */
	private function upload_watermark()
	{
		if ($this->diafan->admin_roles('config'))
		{
			$this->result["error"] = _ADMIN_NO_RULES_FOR_SAVE;
			$this->send_json();
		}

		if (! isset($_FILES["watermark"]) || ! is_array($_FILES["watermark"]) || $_FILES["watermark"]['name'] == '')
		{
			$this->result["errors"]["watermark"] = _ADMIN_NO_FILE_FOR_UPLOAD;
			$this->send_json();
		}

		include_once(ABSOLUTE_PATH."includes/image.php");

		if (! is_dir(ABSOLUTE_PATH.USERFILES.'/watermark'))
		{
			mkdir(ABSOLUTE_PATH.USERFILES.'/watermark', 0777);
		}

		$new_name = strtolower($this->diafan->translit($_FILES["watermark"]['name']));
		$extension = substr(strrchr($new_name, '.'), 1);
		$mimes = array(
			'gif' => 'image/gif',
			'jpeg' => 'image/jpeg',
			'jpg' => 'image/jpeg',
			'jpe' => 'image/jpeg',
			'png' => 'image/png'
		);
		if (file_exists(ABSOLUTE_PATH.USERFILES."/watermark/".$new_name))
		{
			$new_name = rand(0, 999).$new_name;
		}

		if (! in_array($extension, array_keys($mimes)) || empty($mimes[$extension]) || $mimes[$extension] != strtolower($_FILES["watermark"]['type']))
		{
			return '<br>'.sprintf(_LANG_ATTACHMENT_EXTENSIONS, 'gif, jpeg, jpg, jpe, png');
		}

		if (! copy($_FILES["watermark"]['tmp_name'], ABSOLUTE_PATH.USERFILES."/watermark/".$new_name))
		{
			$this->diafan->result["watermark"]["image"] = _ADMIN_NO_UPLOAD_FILE;
			$this->send_json();
		}

		chmod(ABSOLUTE_PATH.USERFILES."/watermark/".$new_name, 0755);

		DB::query("DELETE FROM {config} WHERE name='watermark_path' AND module_name='%h' AND site_id='%d'", $this->diafan->module, $this->diafan->site);
		DB::query("INSERT INTO {config} (module_name, name, value, site_id) VALUES ('%h', 'watermark_path', '%h', '%d')",
				  $this->diafan->module, $new_name, $this->diafan->site);

		include_once(ABSOLUTE_PATH.'modules/images/admin/images.admin.view.php');

		$this->result["data"]   = Images_admin_view::show_watermark($this);
		$this->result["target"] = ".images_container";

		$this->send_json();
	}

	/**
	 * Удаляет водяной знак
	 * 
	 * @return void
	 */
	private function delete_watermark()
	{
		if ($this->diafan->admin_roles('config'))
		{
			$this->result["error"] = _ADMIN_NO_RULES_FOR_DELETE;
			$this->send_json();
		}
		if (empty($_POST["file"]))
		{
			$this->result["error"] = _ADMIN_NO_RULES_FOR_DELETE;
			$this->send_json();
		}
		if (preg_match('/[^a-z0-9\._-]+/', $_POST["file"]))
		{
			$this->result["error"] = _ADMIN_NO_RULES_FOR_DELETE;
			$this->send_json();
		}
		@unlink(ABSOLUTE_PATH.USERFILES.'/watermark/'.$_POST["file"]);

		include_once(ABSOLUTE_PATH.'modules/images/admin/images.admin.view.php');

		$this->result["data"]   = Images_admin_view::show_watermark($this);
		$this->result["target"] = ".images_container";
		DB::query("DELETE FROM {config} WHERE name='watermark_path' AND value='%h'", $_POST["file"]);

		$this->send_json();
	}

	/**
	 * Изменяет водяной знак
	 * 
	 * @return void
	 */
	private function change_watermark()
	{
		if ($this->diafan->admin_roles('config'))
		{
			$this->result["error"] = _ADMIN_NO_RULES_FOR_DELETE;
			$this->send_json();
		}
		if (empty($_POST["file"]))
		{
			$this->result["error"] = _ADMIN_NO_RULES_FOR_DELETE;
			$this->send_json();
		}
		if (preg_match('/[^a-z0-9\._-]+/', $_POST["file"]))
		{
			$this->result["error"] = _ADMIN_NO_RULES_FOR_DELETE;
			$this->send_json();
		}

		DB::query("DELETE FROM {config} WHERE name='watermark_path' AND module_name='%h' AND site_id='%d'", $this->diafan->module, $this->diafan->site);
		DB::query("INSERT INTO {config} (module_name, name, value, site_id) VALUES ('%h', 'watermark_path', '%h', '%d')",
				  $this->diafan->module, $_POST["file"], $this->diafan->site);

		include_once(ABSOLUTE_PATH.'modules/images/admin/images.admin.view.php');

		$this->result["data"]   = Images_admin_view::show_watermark($this);
		$this->result["target"] = ".images_container";

		$this->send_json();
	}

	/**
	 * Изменяет размер всех загруженных изображений
	 * 
	 * @return void
	 */
	private function resize()
	{
		if ($this->diafan->admin_roles('config') || empty($_POST["values"]))
		{
			$this->result["error"] = _ADMIN_NO_RULES_FOR_SAVE;
			$this->send_json();
		}
		$fields = array('images', 'images_cat', 'big_width', 'big_height', 'big_quality', 'medium_width', 'medium_height', 'medium_quality', 'big_width_cat', 
		                'big_height_cat', 'big_quality_cat', 'medium_width_cat', 'medium_height_cat', 'medium_quality_cat');
		foreach ($fields as $field)
		{
			$values[$field] = '';
			if (preg_match('/'.$field.'=([^&]+)&/', $_POST["values"], $m))
			{
				$values[$field] = $m[1];
			}
		}
		if ($values["images"] && (! $values["big_width"] || ! $values["big_height"] || ! $values["big_quality"])
		|| $values["images_cat"] && (! $values["big_width_cat"] || ! $values["big_height_cat"] || ! $values["big_quality_cat"]))
		{
			$this->result["error"] = _ADMIN_NO_CONFIG_BIG_SIZE;
			$this->send_json();
		}
		if ($values["images"] && (! $values["medium_width"] || ! $values["medium_height"] || ! $values["medium_quality"])
		|| $values["images_cat"] && (! $values["medium_width_cat"] || ! $values["medium_height_cat"] || ! $values["medium_quality_cat"]))
		{
			$this->result["error"] = _ADMIN_NO_CONFIG_MEDIUM_SIZE;
			$this->send_json();
		}

		include_once(ABSOLUTE_PATH."includes/image.php");

		$result = DB::query("SELECT id, name, module_name FROM {images} WHERE module_name='%s' OR module_name='%s'", $this->diafan->module, $this->diafan->module."cat");
		while ($row = DB::fetch_array($result))
		{
			if (! file_exists(ABSOLUTE_PATH.USERFILES."/".$this->diafan->module.'/'.$row["name"]))
			{
				DB::query("DELETE FROM {images} WHERE id=%d", $row["id"]);
				continue;
			}

			list($width, $height) = getimagesize(ABSOLUTE_PATH.USERFILES."/".$this->diafan->module.'/'.$row["name"]);
			list($width_m, $height_m) = getimagesize(ABSOLUTE_PATH.USERFILES."/".$this->diafan->module.'/medium/'.$row["name"]);
			if ($row["module_name"] == $this->diafan->module)
			{
				$big_width = $values["big_width"];
				$big_height = $values["big_height"];
				$big_quality = $values["big_quality"];
				$medium_width = $values["medium_width"];
				$medium_height = $values["medium_height"];
				$medium_quality = $values["medium_quality"];
			}
			else
			{
				$big_width = $values["big_width_cat"];
				$big_height = $values["big_height_cat"];
				$big_quality = $values["big_quality_cat"];
				$medium_width = $values["medium_width_cat"];
				$medium_height = $values["medium_height_cat"];
				$medium_quality = $values["medium_quality_cat"];
			}
			if ($width > $big_width || $height > $big_height)
			{
				Image::resize(ABSOLUTE_PATH.USERFILES."/".$this->diafan->module.'/'.$row["name"], $big_width, $big_height, $big_quality);
			}
			if ($width_m > $medium_width || $height_m > $medium_height)
			{
				Image::resize(ABSOLUTE_PATH.USERFILES."/".$this->diafan->module.'/medium/'.$row["name"], $medium_width, $medium_height, $medium_quality);
			}
			elseif ($width_m < $medium_width && $height_m < $medium_height && ($width > $medium_width || $height > $medium_height || $width > $width_m || $height > $height_m))
			{
				copy(ABSOLUTE_PATH.USERFILES."/".$this->diafan->module.'/'.$row["name"],
				     ABSOLUTE_PATH.USERFILES."/".$this->diafan->module."/medium/".$row["name"]);
				Image::resize(ABSOLUTE_PATH.USERFILES."/".$this->diafan->module.'/medium/'.$row["name"], $medium_width, $medium_height, $medium_quality);
			}
		}
		$this->result["error"] = _ADMIN_RESIZE_CONFIG_SUCCESS;

		$this->send_json();
	}

	/**
	 * Отдает ответ Ajax
	 * 
	 * @return void
	 */
	private function send_json()
	{
		if ($this->result)
		{
			include_once(ABSOLUTE_PATH.'plugins/json.php');
			echo to_json($this->result);
			exit;
		}
	}

	/**
	 * Возвращает номер элемента, к которому подключается фотография или тэг
	 * 
	 * @return integer
	 */
	private function get_id_element()
	{
		if (! empty($_POST["id"]))
		{
			return intval($_POST["id"]);
		}
		else
		{
			$names  = array();
			$values = array();
			if ($this->config_module["parent"])
			{
				$names[]  = "parent_id";
				$values[] = "'".$this->diafan->parent."'";
			}
			if ($this->config_module["element"])
			{
				$names[]  = "cat_id";
				$values[] = "'".$this->diafan->cat."'";
			}
			if ($this->config_module["element_site"])
			{
				$names[]  = "site_id";
				$values[] = "'".$this->diafan->site."'";
			}

			DB::query("INSERT INTO {".$this->table."} (".implode(', ', $names).") VALUES (".implode(', ', $values).")");

			return DB::next_id($this->table);
		}
	}
}