<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.4
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include(dirname(dirname(dirname(__FILE__))).'/includes/404.php');
}

/**
 * Order_model
 *
 * Модель модуля "Оформление заказа"
 */
class Order_model extends Model
{
	/**
	 * @var array сгенерированные в моделе данные, передаваемые в шаблон
	 */
	protected $result;

	/**
	 * @var object основной объект системы
	 */
	protected $diafan;

	/**
	 * Конструктор класса
	 * 
	 * @return void
	 */
	public function __construct(&$diafan)
	{
		$this->diafan = &$diafan;
	}

	/**
	 * Генерирует данные для первого шага в офрмлении заказа
	 * 
	 * @return array|boolean
	 */
	public function show_step_1()
	{
		$this->diafan->hide_previous_next = TRUE;

		if ($this->diafan->configmodules('security_user', 'shop') && ! $this->diafan->_user->id)
		{
			$this->diafan->redirect($this->diafan->_route->module('basket'));
			return FALSE;
		}

		if (! DB::query_result("SELECT id FROM {site} WHERE addmodule='registration' AND [act]='1' AND trash='0' AND block='0'"))
		{
			return $this->show_step_2();
		}
		if (empty($_SESSION["basket"]) || !is_array($_SESSION["basket"]))
		{
			$this->diafan->redirect($this->diafan->_route->module('basket'));
			return FALSE;
		}

		include_once(ABSOLUTE_PATH.'modules/basket/basket.model.php');

		$model = new Basket_model($this->diafan);
		$this->result = $model->form();

		$this->result["to-step-2"] = BASE_PATH_HREF.$this->diafan->_route->link($this->diafan->cid, "order").'step2/';

		$registration = array("fio" => '', "mail" => '', "name" => '', "captcha" => '', "user" => '');

		$registration["action"]         = BASE_PATH_HREF.$this->diafan->_route->module('registration');
		$registration["error"]          = $this->get_error("registration");
		$registration["error_name"]     = $this->get_error("registration", 'name');
		$registration["error_fio"]      = $this->get_error("registration", 'fio');
		$registration["error_password"] = $this->get_error("registration", 'password');
		$registration["error_mail"]     = $this->get_error("registration", 'mail');
		$registration["user_id"]        = $this->diafan->_user->id;
		$registration["url"] 	        = $this->result["to-step-2"];

		if ($this->diafan->configmodules('security_code', "usersite"))
		{
			$registration["captcha"] = $this->diafan->_captcha->get("registration", $this->get_error("registration", "code"));
		}
		$registration["use_avatar"] = $this->diafan->configmodules("avatar", "usersite");
		if ($registration["use_avatar"])
		{
			if ($this->diafan->_user->id)
			{
				$registration["avatar"] = file_exists(ABSOLUTE_PATH.'userfiles/avatar/'.$this->diafan->_user->name.'.png');
			}
			$registration["avatar_width"]  = $this->diafan->configmodules("avatar_width", "usersite");
			$registration["avatar_height"] = $this->diafan->configmodules("avatar_height", "usersite");
			$registration["error_avatar"]  = $this->get_error("registration", 'avatar');
		}
		$this->result["registration"] = $registration;
		$show_login = array("error" => $this->diafan->_user->errauth ? _LANG_WRONG_LOGIN_OR_PASS : '', "action" => '', "user" => '', 'hide' => TRUE);

		$this->result["show_login"] = $show_login;

		return $this->result;
	}

	/**
	 * Генерирует данные для второго шага в офрмлении заказа
	 * 
	 * @return array|boolean
	 */
	public function show_step_2()
	{
		$this->diafan->hide_previous_next = TRUE;

		if ($this->diafan->configmodules('security_user', 'shop') && ! $this->diafan->_user->id)
		{
			$this->diafan->redirect($this->diafan->_route->module('basket'));
			return FALSE;
		}

		if (empty($_SESSION["basket"]) || ! is_array($_SESSION["basket"]))
		{
			$this->diafan->redirect($this->diafan->_route->module('basket'));
			return FALSE;
		}

		include_once(ABSOLUTE_PATH.'modules/basket/basket.model.php');

		$model = new Basket_model($this->diafan);
		$this->result = $model->form();

		$cache_meta = array(
				    "name"     => "order_param",
				    "language" => _LANG
				    );
		if (! $rows_param = $this->diafan->_cache->get($cache_meta, "shop"))
		{
			$rows_param = array();
			$result = DB::query("SELECT id, [name], type, required, [text] FROM {shop_order_param} WHERE trash='0' ORDER BY sort ASC");

			while ($row = DB::fetch_array($result))
			{
				if ($row["type"] == 'select' || $row["type"] == 'multiple')
				{
					$result_select = DB::query("SELECT [name], id FROM {shop_order_param_select} WHERE param_id=%d ORDER BY sort ASC", $row["id"]);
					while ($row_select = DB::fetch_array($result_select))
					{
						$row["select_array"][] = $row_select;
					}
				}
				$rows_param[] = $row;
			}
			//сохранение кеша
			$this->diafan->_cache->save($rows_param, $cache_meta, "shop");
		}
		$this->result["rows_param"] = $rows_param;
		$multiple = array();
		foreach ($rows_param as $row)
		{
			$this->result['error_p'.$row["id"]] = $this->get_error("order", 'p'.$row["id"]);
			if ($row["type"] == "multiple")
			{
				$multiple[] = $row["id"];
			}
		}

		if ($this->diafan->_user->id)
		{
			$max_order_id = DB::query_result("SELECT MAX(id) FROM {shop_order} WHERE user_id=%d AND trash='0'", $this->diafan->_user->id);
			$result = DB::query("SELECT value, param_id FROM {shop_order_param_element} WHERE trash='0' AND element_id=%d", $max_order_id);
			while ($row = DB::fetch_array($result))
			{
				if (in_array($row["param_id"], $multiple))
				{
					$this->result["user"]['p'.$row["param_id"]][] = $row["value"];
				}
				else
				{
					$this->result["user"]['p'.$row["param_id"]] = $row["value"];
				}
			}
		}

		$this->result["text"]  = $this->diafan->configmodules("order_step_2_text", "shop");

		$this->result["error"] = $this->get_error("order");

		return $this->result;
	}

	/**
	 * Генерирует данные для третьего шага в офрмлении заказа
	 * 
	 * @return array|boolean
	 */
	public function show_step_3()
	{
		$this->diafan->hide_previous_next = TRUE;

		if ($this->diafan->configmodules('security_user', 'shop') && ! $this->diafan->_user->id)
		{
			$this->diafan->redirect($this->diafan->_route->module('basket'));
			return FALSE;
		}

		if (empty($_SESSION["order"]) || ! is_array($_SESSION["order"]) || empty($_SESSION["order_pay"]))
		{
			$this->diafan->redirect($this->diafan->_route->module('basket'));
			return FALSE;
		}

		$this->result["webmoney"] = $this->diafan->configmodules("webmoney", "shop") ? 1 : 0;
		$this->result["robox"]    = $this->diafan->configmodules("robox", "shop") ? 1 : 0;
		$this->result["non_cash"] = $this->diafan->configmodules("non_cash", "shop") ? 1 : 0;
		$this->result["courier"]  = $this->diafan->configmodules("courier", "shop") ? 1 : 0;

		return $this->result;
	}

	/**
	 * Генерирует данные для четвертого шага в офрмлении заказа
	 * 
	 * @return array|boolean
	 */
	public function show_step_4()
	{
		$this->diafan->hide_previous_next = TRUE;

		if ($this->diafan->configmodules('security_user', 'shop') && ! $this->diafan->_user->id)
		{
			$this->diafan->redirect($this->diafan->_route->module('basket'));
			return FALSE;
		}

		$row = DB::fetch_array(DB::query("SELECT payment, summ FROM {shop_order} WHERE id=%d", $this->diafan->show));
		if ($row["payment"] == 1 && $this->diafan->configmodules("webmoney", "shop"))
		{
			$this->webmoney($row["summ"]);
		}
		elseif ($row["payment"] == 2 && $this->diafan->configmodules("robox", "shop"))
		{
			$this->roboxchange($row["summ"]);
		}
		elseif ($row["payment"] == 3 && $this->diafan->configmodules("non_cash", "shop"))
		{
			$this->non_cash();
		}
		else
		{
			include(ABSOLUTE_PATH.'includes/404.php');
		}
		return $this->result;
	}

	/**
	 * Генерирует данные для пятого шага в офрмлении заказа
	 * 
	 * @return array
	 */
	public function show_step_5()
	{
		$this->diafan->hide_previous_next = TRUE;

		if ($this->diafan->step == 5)
		{
			return $this->diafan->configmodules("payment_success_text", "shop");
		}
		else
		{
			return $this->diafan->configmodules("payment_fail_text", "shop");
		}
	}

	/**
	 * Платежная система WebMoney
	 *
	 * @param string $summ сумма заказа
	 * @return boolean true
	 */
	private function webmoney($summ)
	{
		$this->result["type"]      = "webmoney";
		$this->result["desc"]      = str_replace('%order', $this->diafan->show, $this->diafan->configmodules("desc_order", "shop"));
		$this->result["wm_target"] = WM_TARGET;
		
		$this->result["summ"]      = $summ;
		$this->result["order_id"]  = $this->diafan->show;

		$this->result["rnd"]       = strtoupper(substr(md5(uniqid(microtime(), 1)).getmypid(), 1, 8));

		// режим тестирования:
		//  0 или не отсутствует: Для всех тестовых платежей сервис будет имитировать успешное выполнение;
		//  1: Для всех тестовых платежей сервис будет имитировать выполнение с ошибкой (платеж не выполнен);
		//  2: Около 80% запросов на платеж будут выполнены успешно, а 20% - не выполнены.
		// $result["LMI_SIM_MODE"]=2;

		$this->result["text"] = $this->diafan->configmodules("wm_text", "shop");

		DB::query("INSERT INTO {shop_payment} (order_id, state, RND, timestamp) VALUES('%d', 'I', '%s', CURRENT_TIMESTAMP())",
		          $this->diafan->show, $this->result["rnd"]);
		return TRUE;
	}

	/**
	 * Платежная система Roboxchange
	 * 
	 * @param string $summ сумма заказа
	 * @return boolean true
	 */
	private function roboxchange($summ)
	{
		$this->result["type"]      = "roboxchange";
		// регистрационная информация 
		$this->result["login"]    = ROBOX_LOGIN; //логин 
		$mrh_pass1                = ROBOX_PASS_1; //пароль1
		$this->result["summ"]     = $summ;
		$this->result["order_id"] = $this->diafan->show;

		//описание покупки
		$this->result["desc"]     = str_replace('%order', $this->diafan->show, $this->diafan->configmodules("desc_order", "shop"));
		$this->result["desc"]     = $this->diafan->translit($this->result["desc"]);

		//формирование подписи
		$this->result["crc"]      = md5($this->result["login"].":".$summ.":".$this->diafan->show.":".$mrh_pass1);

		$link = "https://merchant.roboxchange.com/Index.aspx?MrchLogin=".$this->result["login"]."&OutSum="
		.$this->result["summ"]."&InvId=".$this->result["order_id"]."&Desc=".$this->result["desc"]."&SignatureValue=".$this->result["crc"];

		$this->diafan->redirect($link);
		exit;

		//$this->result["text"]     = $this->diafan->configmodules("robox_text", "shop");

		return TRUE;
	}

	/**
	 * Способ оплаты: Банковский платеж
	 * 
	 * @return string
	 */
	private function non_cash()
	{
		$this->result["type"]     = "non_cash";
		$this->result["text"]     = $this->diafan->configmodules("non_cash_text", "shop");
		$this->result["order_id"] = $this->diafan->show;
		return TRUE;
	}
}