<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.4
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include(dirname(dirname(dirname(__FILE__))).'/includes/404.php');
}

/**
 * Rating_ajax
 *
 * Обработка запроса при оценке элемента
 */
class Rating_ajax
{
	/**
	 * @var object основной объект системы
	 */
	public $diafan;

	/**
	 * @var array внутренние сообщения
	 */
	private $result;

	/**
	 * @var string название модуля, элемент которого оценивается
	 */
	private $module_name;

	/**
	 * @var integer номер элемента, который оценивается
	 */
	private $element_id;

	/**
	 * Конструктор класса
	 * 
	 * @return void
	 */
	public function __construct(&$diafan)
	{
		$this->diafan = &$diafan;
	}

	/**
	 * Обрабатывает полученные данные из формы
	 * 
	 * @return boolean
	 */
	public function ajax_request()
	{
		if (! empty($_POST['module']) && $_POST['module'] == 'rating' && ! empty($_POST['rating']) && ! empty($_POST['element_id']) && ! empty($_POST['module_name']))
		{
			$this->element_id  = intval($_POST["element_id"]);
			$this->module_name = preg_replace('/[^a-zA-Z0-9-_]+/', '', $_POST['module_name']);

			if ($this->check_element())
				return TRUE;

			if ($this->check_roles())
				return TRUE;

			if ($this->check_captcha())
				return TRUE;

			if ($this->check_log())
				return TRUE;

			if (DB::query_result("SELECT COUNT(id) FROM {rating} WHERE element_id='%d' AND module_name='%s'",
					    $this->element_id, $this->module_name)
			   )
			{ 
				DB::query("UPDATE {rating} SET rating=(rating*count_votes+%d)/(count_votes+1), count_votes=count_votes+1,created='%d'"
					     ." WHERE element_id='%d' AND module_name='%s'",
					     $_POST["rating"],
					     time(),
					     $this->element_id,
					     $this->module_name
					     );
				$rating = DB::query_result("SELECT rating FROM {rating} WHERE element_id='%d' AND module_name='%s'", $this->element_id, $this->module_name);
			}
			else
			{			
				DB::query("INSERT INTO {rating} (rating, count_votes, element_id, module_name, created) VALUES ('%d', 1, '%d', '%s', '%d')",
				          $_POST["rating"],
				          $this->element_id,
				          $this->module_name,
				          time()
				         );
				$rating = intval($_POST["rating"]);
			}

			$result = array(
				"element_id"  => $this->element_id,
				"module_name" => $this->module_name,
				"rating"      => round($rating),
				"new"         => 1
			);

			$this->result["target"] = "#rating".$this->element_id.$this->module_name.'-result';
			$this->result["data"]   = $this->diafan->_tpl->get('get', 'rating', $result);
			$this->result["form_hide"] = 1;

			return $this->send_errors();
		}
	}

	/**
	 * Проверяет существует ли оцениваемый элемент
	 * 
	 * @return boolean
	 */
	private function check_element()
	{
		if (! $this->element_id || ! $this->module_name)
		{
			return TRUE;
		}
		if (! DB::query_result("SELECT id FROM {".$this->module_name."} WHERE id=%d", $this->element_id))
		{
			return TRUE;
		}
	}

	/**
	 * Проверяет права доступа
	 * 
	 * @return boolean
	 */
	private function check_roles()
	{
		if ($this->diafan->configmodules('security_user', 'rating') && ! $this->diafan->_user->id)
		{
			$this->result["errors"][0] = _LANG_VOTING_CLOSE_UNREGISTRED;
		}

		return $this->send_errors();
	}

	/**
	 * Проверяет правильность каптчи
	 * 
	 * @return boolean
	 */
	private function check_captcha()
	{
		$this->result["code"] = '';
		if ($this->diafan->configmodules('security', 'rating') == 2)
		{
			$mes = $this->diafan->_captcha->is_right('rating'.$this->element_id.$this->module_name);
			$this->result["code"] = $this->diafan->_captcha->get('rating'.$this->element_id.$this->module_name, '', TRUE);				
			if ($mes)
			{
				$this->result["errors"]["code"] = $mes;
			}
		}
		return $this->send_errors();
	}

	/**
	 * Проверяет попытку голосовать повторно
	 * 
	 * @return boolean
	 */
	private function check_log()
	{
		if (! session_id())
		{
			$this->result["errors"][0] = _LANG_VOTE_SECOND;
		}
		if ($this->diafan->configmodules('security', 'rating') == 3)
		{
			if (DB::query_result("SELECT COUNT(id) FROM {log_note} WHERE session_id='%s' AND element_id='%d' AND module_name='%s' AND include_name='rating'",
			                    $this->diafan->configmodules('security_user', 'rating') ? $this->diafan->_user->id : session_id(),
			                    $this->element_id, $this->module_name
			                   ))
			{
				$this->result["errors"][0] = _LANG_VOTE_SECOND;
			}
			else
			{
				
				DB::query("INSERT INTO {log_note} (include_name, element_id, note, created, ip, session_id, module_name) "
				          ." VALUES ('rating', '%d', '%d', %d, '%s', '%s', '%s')",
				          $this->element_id,
				          $_POST["ajax_rating"],
				          time(),
				          getenv('REMOTE_ADDR'),
				          $this->diafan->configmodules('security_user', 'rating') ? $this->diafan->_user->id : session_id(),
				          $this->module_name
				         );
			}
		}

		if ($this->diafan->configmodules('security', 'rating') == 4)
		{
			if (! empty($_SESSION["rating"][$this->element_id.$this->module_name]))
			{
				$this->result["errors"][0] = _LANG_VOTE_SECOND;
			}
			else
			{
				$_SESSION["rating"][$this->element_id.$this->module_name] = 1;
			}
		}

		return $this->send_errors();
	}

	/**
	 * Отправляет несколько ошибок
	 *
	 * @return boolean
	 */
	private function send_errors()
	{
		if (! empty($this->result["errors"]) || ! empty($this->result["data"]))
		{
			if (! empty($_POST['ajax']))
			{
				include_once(ABSOLUTE_PATH.'plugins/json.php');
				echo to_json($this->result);
			}
			else
			{
				$query = '';
				if (! empty($this->result["errors"]))
				{
					foreach ($this->result["errors"] as $field => $mes)
					{
						if (empty($_POST["update"]))
						{
							$query .= '&mess'.($field ? '-'.$field : '').'='.$mes;
						}
					}
				}
				$this->diafan->redirect($this->diafan->_route->current_link().($query ? '?error=rating'.$query : ''));
			}
			return TRUE;
		}
		return FALSE;
	}
}