<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.4
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include(dirname(dirname(dirname(__FILE__))).'/includes/404.php');
}

/**
 * Shop_admin_ajax
 * 
 * Обработка Ajax-запросов в административной части модуля Магазин
 */
class Shop_admin_ajax
{
	/**
	 * @var object основной объект системы
	 */
	public $diafan;

	/**
	 * @var array результаты, передаваемы Ajaxом
	 */
	private $result;

	/**
	 * Конструктор класса
	 * 
	 * @return void
	 */
	public function __construct($frame)
	{
		$this->diafan = $frame->diafan;

		// Прошел ли пользователь проверку идентификационного хэша
		if ($_POST["action"] != "show_rel_goods")
		{
			if (! $this->diafan->_user->checked)
			{
				$this->result["error"] = 'ERROR';
				$this->send_json();
			}
		}
		$this->result["hash"] = $this->diafan->_user->get_hash();

		if (! empty($_POST["action"]))
		{
			switch($_POST["action"])
			{
				case 'up_param':
				case 'down_param':
					$this->sort_param();
					break;
				case 'delete_param':
					$this->delete_param();
					break;
				case 'rel_goods':
					$this->rel_goods();
					break;
				case 'show_rel_goods':
					$this->show_rel_goods();
					break;
				case 'delete_rel_good':
					$this->delete_rel_good();
					break;
				case 'show_discount_goods':
					$this->show_discount_goods();
					break;
				case 'discount_good':
					$this->discount_good();
					break;
				case 'delete_discount_good':
					$this->delete_discount_good();
					break;
			}
		}
	}

	/**
	 * Сортирует значения списка параметров
	 * 
	 * @return void
	 */
	private function sort_param()
	{
		$id = intval($_POST['id']);
		if (! empty($id))
		{
			$oldsort = DB::query_result("SELECT sort FROM {shop_param_select} WHERE param_id=%d AND id=%d", $this->diafan->edit, $id);
			$row = DB::fetch_array(DB::query("SELECT id, sort FROM {shop_param_select} WHERE param_id=%d AND id<>%d AND"
				  .($_POST["action"] == "up_param" ? " sort<=%d ORDER BY sort DESC" : " sort>=%d ORDER BY sort ASC")." LIMIT 1",
				  $this->diafan->edit, $id, $oldsort));
			if ($row)
			{
				DB::query("UPDATE {shop_param_select} SET sort=%d WHERE id=%d", $oldsort, $row["id"]);
				DB::query("UPDATE {shop_param_select} SET sort=%d WHERE id=%d", $row["sort"], $id);
			}
		}
		$this->diafan->_cache->delete("", $this->diafan->module);

		$this->send_json();
	}

	/**
	 * Удаляет значения списка параметров
	 * 
	 * @return void
	 */
	private function delete_param()
	{
		$id = intval($_POST['id']);
		if (! empty($id))
		{
			DB::query("DELETE FROM {shop_param_select} WHERE param_id=%d AND id=%d", $this->diafan->edit, $id);
		}
		$this->diafan->_cache->delete("", $this->diafan->module);

		$this->send_json();
	}

	/**
	 * Прикрепляет похожие товары
	 * 
	 * @return void
	 */
	private function rel_goods()
	{
		if (empty($_POST["element_id"]))
		{
			DB::query("INSERT INTO {shop} () VALUES ()");
			$_POST["element_id"] = DB::next_id("shop");
			$this->result["id"] = $_POST["element_id"];
		}
		include_once(ABSOLUTE_PATH.'modules/shop/admin/shop.admin.view.php');
		if ($_POST["element_id"] != $_POST["rel_id"] &&
		   ! DB::query_result("SELECT COUNT(*) FROM {shop_rel} WHERE good_id=%d AND rel_good_id=%d OR rel_good_id=%d AND good_id=%d",
		                      $_POST["element_id"], $_POST["rel_id"], $_POST["element_id"], $_POST["rel_id"]))
		{
			DB::query("INSERT INTO {shop_rel} (good_id, rel_good_id) VALUES (%d, %d)", $_POST["element_id"], $_POST["rel_id"]);
		}

		$this->result["data"] = Shop_admin_view::rel_goods($this, $_POST["element_id"]);

		$this->send_json();
	}

	/**
	 * Подгружает список товаров для выбора похожих
	 * 
	 * @return void
	 */
	private function show_rel_goods()
	{
		if (empty($_POST["element_id"]))
		{
			$_POST["element_id"] = 0;
		}
		$nastr = 24;
		$list = '';
		if (empty($_POST["page"]))
		{
			$start = 0;
			$page = 1;
			if (! isset($_POST["search"]))
			{
				$list = _LANG_SEARCH.': <input type="text" size="30" class="rel_shop_search">
				<div class="rel_all_goods_container">';
			}
		}
		else
		{
			$page = intval($_POST["page"]);
			$start = ($page - 1) * $nastr; 
		}
		$rel_goods = array();
		if ($_POST["element_id"])
		{
			$result = DB::query("SELECT good_id, rel_good_id FROM {shop_rel} WHERE good_id=%d OR rel_good_id=%d", $_POST["element_id"]);
			while ($row = DB::fetch_array($result))
			{
				$rel_goods[] = $row["good_id"];
				$rel_goods[] = $row["rel_good_id"];
			}
		}

		if (! empty($_POST["search"]))
		{
			$count = DB::query_result("SELECT COUNT(*) FROM {shop} WHERE trash='0' AND name LIKE '%h%%' AND id<>%d", $_POST["search"], $_POST["element_id"]);
			$result = DB::query_range("SELECT id, name FROM {shop} WHERE trash='0' AND name LIKE '%h%%' AND id<>%d", $_POST["search"], $_POST["element_id"], $start, $nastr);
		}
		else
		{
			$count = DB::query_result("SELECT COUNT(*) FROM {shop} WHERE trash='0' AND id<>%d", $_POST["element_id"]);
			$result = DB::query_range("SELECT id, name FROM {shop} WHERE trash='0' AND id<>%d", $_POST["element_id"], $start, $nastr);
		}
		while ($row = DB::fetch_array($result))
		{
			$img = DB::query_result("SELECT name FROM {images} WHERE element_id=%d AND module_name='shop' AND trash='0' ORDER BY sort ASC LIMIT 1", $row["id"]);
			$list .= '<div class="rel_shop" element_id="'.$row["id"].'">
			<div'.(in_array($row["id"], $rel_goods) ? ' class="rel_shop_selected"' : '').'>
			'.($img ? '<a href="#"><img src="'.BASE_PATH.USERFILES.'/shop/small/'.$img.'"></a><br>' : '').'
			<a href="#">'.$row["name"].'</a>
			</div>
			</div>';
		}
		$list .= '<div class="clearing rel_shop_navig">';
		for ($i = 1; $i <= ceil($count/$nastr); $i++)
		{
			if ($i != $page)
			{
				$list .= '<a href="#" page="'.$i.'">'.$i.'</a> ';
			}
			else
			{
				$list .= '['.$i.'] ';
			}
		}
		$list .= '</div>';
		if (empty($_POST["page"]) && ! isset($_POST["search"]))
		{
			$list .= '</div>';
		}

		$this->result["data"] = $list;

		$this->send_json();
	}

	/**
	 * Удаляет похожие товары
	 * 
	 * @return void
	 */
	private function delete_rel_good()
	{
		DB::query("DELETE FROM {shop_rel} WHERE good_id=%d AND rel_good_id=%d", $_POST['element_id'], $_POST['rel_id']);
		DB::query("DELETE FROM {shop_rel} WHERE rel_good_id=%d AND good_id=%d", $_POST['element_id'], $_POST['rel_id']);

		$this->diafan->_cache->delete("", $this->diafan->module);

		$this->send_json();
	}

	/**
	 * Подгружает список товаров
	 * 
	 * @return void
	 */
	private function show_discount_goods()
	{
		if (empty($_POST["discount_id"]))
		{
			$_POST["discount_id"] = 0;
		}
		$nastr = 24;
		$list = '';
		if (empty($_POST["page"]))
		{
			$start = 0;
			$page = 1;
			if (! isset($_POST["search"]))
			{
				$list = _LANG_SEARCH.': <input type="text" size="30" class="rel_shop_search">
				<div class="rel_all_goods_container">';
			}
		}
		else
		{
			$page = intval($_POST["page"]);
			$start = ($page - 1) * $nastr; 
		}
		$discount_goods = array();
		if ($_POST["discount_id"])
		{
			$result = DB::query("SELECT good_id FROM {shop_discount_object} WHERE discount_id=%d", $_POST["discount_id"]);
			while ($row = DB::fetch_array($result))
			{
				$discount_goods[] = $row["good_id"];
			}
		}
		if (! empty($_POST["search"]))
		{
			$count = DB::query_result("SELECT COUNT(*) FROM {shop} WHERE trash='0' AND name LIKE '%h%%'", $_POST["search"]);
			$result = DB::query_range("SELECT id, name FROM {shop} WHERE trash='0' AND name LIKE '%h%%'", $_POST["search"], $start, $nastr);
		}
		else
		{
			$count = DB::query_result("SELECT COUNT(*) FROM {shop} WHERE trash='0'");
			$result = DB::query_range("SELECT id, name FROM {shop} WHERE trash='0'", $start, $nastr);
		}
		while ($row = DB::fetch_array($result))
		{
			$img = DB::query_result("SELECT name FROM {images} WHERE element_id=%d AND module_name='shop' AND trash='0' ORDER BY sort ASC LIMIT 1", $row["id"]);
			$list .= '<div class="rel_shop" element_id="'.$row["id"].'">
			<div'.(in_array($row["id"], $discount_goods) ? ' class="rel_shop_selected"' : '').'>
			'.($img ? '<a href="#"><img src="'.BASE_PATH.USERFILES.'/shop/small/'.$img.'"></a><br>' : '').'
			<a href="#">'.$row["name"].'</a>
			</div>
			</div>';
		}
		$list .= '<div class="clearing rel_shop_navig">';
		for ($i = 1; $i <= ceil($count/$nastr); $i++)
		{
			if ($i != $page)
			{
				$list .= '<a href="#" page="'.$i.'">'.$i.'</a> ';
			}
			else
			{
				$list .= '['.$i.'] ';
			}
		}
		$list .= '</div>';
		if (empty($_POST["page"]) && ! isset($_POST["search"]))
		{
			$list .= '</div>';
		}

		$this->result["data"] = $list;

		$this->send_json();
	}

	/**
	 * Прикрепляет скидку к товару
	 * 
	 * @return void
	 */
	private function discount_good()
	{
		if (! $_POST["discount_id"])
		{
			DB::query("INSERT INTO {shop_discount} () VALUES ()");
			$_POST["discount_id"] = DB::next_id("shop_discount");
			$this->result["id"] = $_POST["discount_id"];
		}
		include_once(ABSOLUTE_PATH.'modules/shop/admin/shop.admin.view.php');
		if (! DB::query_result("SELECT COUNT(*) FROM {shop_discount_object} WHERE good_id=%d AND discount_id=%d", $_POST["good_id"], $_POST["discount_id"]))
		{
			DB::query("INSERT INTO {shop_discount_object} (good_id, discount_id) VALUES (%d, %d)", $_POST["good_id"], $_POST["discount_id"]);
		}

		$this->result["data"] = Shop_admin_view::discount_goods($this, $_POST["discount_id"]);

		$this->send_json();
	}

	/**
	 * Удаляет скидку на товар
	 * 
	 * @return void
	 */
	private function delete_discount_good()
	{
		DB::query("DELETE FROM {shop_discount_object} WHERE good_id=%d AND discount_id=%d", $_POST['good_id'], $_POST['discount_id']);

		$this->diafan->_cache->delete("", $this->diafan->module);

		$this->send_json();
	}

	/**
	 * Отдает ответ Ajax
	 * 
	 * @return void
	 */
	private function send_json()
	{
		if ($this->result)
		{
			include_once(ABSOLUTE_PATH.'plugins/json.php');
			echo to_json($this->result);
			exit;
		}
	}
}