<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.4
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include(dirname(dirname(dirname(__FILE__))).'/includes/404.php');
}

/**
 * Shop_model
 *
 * Модель модуля "Магазин"
 */
class Shop_model extends Model
{
	/**
	 * @var array сгенерированные в моделе данные, передаваемые в шаблон
	 */
	protected $result;

	/**
	 * @var object основной объект системы
	 */
	protected $diafan;

	/**
	 * @var array значения параметров, влияющих на цену
	 */
	private $depends;

	/**
	 * @var array направления сортировки товаров
	 */
	private $sort = array(
					1 => 'p.price ASC',
					2 => 'p.price DESC',
					3 => 's.name ASC',
					4 => 's.name DESC'
				);

	/**
	 * @var array скидки
	 */
	private $discounts;

	/**
	 * Конструктор класса
	 * 
	 * @return void
	 */
	public function __construct(&$diafan)
	{
		$this->diafan = &$diafan;
		if ($this->diafan->sort > count($this->sort))
		{
			include(ABSOLUTE_PATH.'includes/404.php');
		}
	}

	/**
	 * Генерирует данные для списка товаров, если отключена группировка товаров
	 * 
	 * @return array
	 */
	public function list_()
	{
		if ($this->diafan->cat)
		{
			include(ABSOLUTE_PATH.'includes/404.php');
		}
		$cache_meta = array(
				"name"     => "list",
				"language" => _LANG,
				"page"     => $this->diafan->page > 1 ? $this->diafan->page : 1,
				"site_id"  => $this->diafan->cid,
				"sort"     => $this->diafan->sort
			);
		//кеширование
		if (! $this->result = $this->diafan->_cache->get($cache_meta, $this->diafan->module))
		{
			$this->result = array();

			////navigation//
			$this->diafan->_paginator->page    = $this->diafan->page;
			$this->diafan->_paginator->navlink = $this->diafan->_route->current_link("page");
			$this->diafan->_paginator->navlink_tpl = $this->diafan->_route->current_link("", array("page" => "%d"));
			$this->diafan->_paginator->nen = DB::query_result("SELECT COUNT(id) FROM {shop} WHERE [act]='1' AND trash='0' AND site_id=%d", $this->diafan->cid);
			$links = $this->diafan->_paginator->get();
			////navigation///

			$result = DB::query_range("SELECT s.id, s.[name], s.timeedit, s.[anons], s.site_id, s.no_buy, s.cat_id, s.article FROM {shop} AS s"
			                          .($this->diafan->sort == 1 || $this->diafan->sort == 2 ? " LEFT JOIN {shop_price} AS p ON p.good_id=s.id AND p.trash='0'" : '')
			                          ." WHERE s.[act]='1' AND s.trash='0' AND s.site_id=%d"
			                          ." ORDER BY ".($this->diafan->sort ? $this->sort[$this->diafan->sort].',' : '')
			                          ." s.sort ASC, s.id ASC", $this->diafan->cid, $this->diafan->_paginator->polog, $this->diafan->_paginator->nastr);
			
			$this->result["rows"] = $this->get_elements($result);

			$this->result["basket_link"] = $this->diafan->_route->module("basket");

			$this->result["paginator"] = $links;

			//сохранение кеша
			$this->diafan->_cache->save($this->result, $cache_meta, $this->diafan->module);
		}
		$this->result["buy"] = ! $this->diafan->configmodules('security_user', "shop") || $this->diafan->_user->id ? TRUE : FALSE;

		if ($this->result["rows"])
		{
			foreach ($this->result["rows"] as $i => $row)
			{
				$this->result["rows"][$i] = $this->select_price($row);

				$this->result["rows"][$i]["tags"] = $this->diafan->configmodules("tags") ? $this->diafan->_tags->get($row["id"]) : '';

				$this->result["rows"][$i]["show_rating"] = $this->diafan->configmodules("rating") ? $this->diafan->_rating->get($row["id"]) : '';
			}
		}
		foreach ($this->sort as $key => $value)
		{
			$this->result["link_sort"][$key] =
			$this->diafan->sort != $key ? $this->diafan->_route->current_link("", array('sort' => $key)) : '';
		}
		$this->result["paginator"] = $this->diafan->_tpl->get('get', 'paginator', $this->result["paginator"]);
		return $this->result;
	}

	/**
	 * Генерирует данные для списка товаров, найденных при помощи поиска
	 * 
	 * @return array
	 */
	public function list_search()
	{
		$this->result = array();

		list($where, $where_param, $values, $getnav) = $this->get_where();

		////navigation//
		$this->diafan->_paginator->page    = $this->diafan->page;
		$this->diafan->_paginator->get_nav = $getnav;
		$this->diafan->_paginator->navlink = $this->diafan->_route->current_link("page");

		$this->diafan->_paginator->nen = DB::query_result("SELECT COUNT(DISTINCT s.id) FROM {shop} as s ".$where_param
		                                                  ." WHERE s.[act]='1' AND s.trash='0' ".$where, $values);
		$links = $this->diafan->_paginator->get();
		////navigation///

		$this->result["path"] = $this->get_path('shop');
		$this->result["titlemodule"] = _LANG_SEARCH_GOODS;
		if (! $this->diafan->_paginator->nen)
		{
			$this->result["err"] = _LANG_IT_IS_NOT_FOUND;
			return $this->result;
		}

		$result = DB::query_range("SELECT s.id, s.[name], s.timeedit, s.[anons], s.cat_id, s.site_id, s.no_buy, s.article FROM {shop} AS s"
		                          .($this->diafan->sort == 1 || $this->diafan->sort == 2 ? " LEFT JOIN {shop_price} AS p ON p.good_id=s.id AND p.trash='0'" : '')
		                          .$where_param
		                          ." WHERE s.[act]='1' AND s.trash='0'".$where
		                          ." GROUP BY s.id"
		                          ." ORDER BY "
		                          .($this->diafan->sort ? $this->sort[$this->diafan->sort].',' : '')
		                          ." sort ASC, id ASC", $values,
		                          $this->diafan->_paginator->polog, $this->diafan->_paginator->nastr);

		$this->result["rows"] = $this->get_elements($result);
		$this->result["buy"]  = ! $this->diafan->configmodules('security_user', "shop") || $this->diafan->_user->id ? TRUE : FALSE;

		$this->result["basket_link"] = $this->diafan->_route->module("basket");

		$this->result["paginator"] = $this->diafan->_tpl->get('get', 'paginator', $links);

		if ($this->result["rows"])
		{
			foreach ($this->result["rows"] as $i => $row)
			{
				$this->result["rows"][$i] = $this->select_price($row);

				$this->result["rows"][$i] = $this->useradmin($this->result["rows"][$i]);

				$this->result["rows"][$i]["tags"] = $this->diafan->configmodules("tags") ? $this->diafan->_tags->get($row["id"]) : '';

				$this->result["rows"][$i]["show_rating"] = $this->diafan->configmodules("rating") ? $this->diafan->_rating->get($row["id"]) : '';
			}
		}

		foreach ($this->sort as $key => $value)
		{
			$this->result["link_sort"][$key] =
			$this->diafan->sort != $key ? $this->diafan->_route->current_link("", array('sort' => $key)).$getnav : '';
		}
		return $this->result;
	}

	/**
	 * Генерирует данные для списка товаров, соответствующих значению доп. характеристики
	 * 
	 * @return array
	 */
	public function list_param()
	{
		$this->result = array();
		$cache_meta = array(
				"name"     => "list_param",
				"language" => _LANG,
				"page"     => $this->diafan->page > 1 ? $this->diafan->page : 1,
				"site_id"  => $this->diafan->cid,
				"sort"     => $this->diafan->sort,
				"param"     => $this->diafan->param
			);
		//кеширование
		if (! $this->result = $this->diafan->_cache->get($cache_meta, $this->diafan->module))
		{
			$param_value = DB::fetch_array(DB::query("SELECT param_id, [name] FROM {shop_param_select} WHERE id=%d", $this->diafan->param));
			if (! $param_value = DB::fetch_array(DB::query("SELECT param_id, [name] FROM {shop_param_select} WHERE id=%d", $this->diafan->param)))
			{
				include(ABSOLUTE_PATH.'includes/404.php');
			}
			if (! $param = DB::fetch_array(DB::query("SELECT [name] FROM {shop_param} WHERE id=%d AND site_id=%d", $param_value["param_id"], $this->diafan->cid)))
			{
				include(ABSOLUTE_PATH.'includes/404.php');
			}
			////navigation//
			$this->diafan->_paginator->page    = $this->diafan->page;
			$this->diafan->_paginator->navlink = $this->diafan->_route->current_link("page");

			$this->diafan->_paginator->nen =
			DB::query_result("SELECT COUNT(DISTINCT s.id) FROM {shop} AS s "
			                 ." INNER JOIN {shop_param_element} AS e ON e.element_id=s.id"
			                 ." WHERE s.[act]='1' AND s.trash='0' AND e.param_id=%d AND e.value=%d AND s.site_id=%d",
			                 $param_value["param_id"], $this->diafan->param, $this->diafan->cid
			                );
			$this->result["paginator"] = $this->diafan->_paginator->get();
			////navigation///

			$this->result["path"] = $this->get_path('shop');
			$this->result["titlemodule"] = $param["name"].': '.$param_value["name"];

			$result = DB::query_range("SELECT s.id, s.[name], s.timeedit, s.[anons], s.cat_id, s.site_id, s.no_buy, s.article FROM {shop} AS s"
			                          .($this->diafan->sort == 1 || $this->diafan->sort == 2 ? " LEFT JOIN {shop_price} AS p ON p.good_id=s.id AND p.trash='0'" : '')
			                          ." INNER JOIN {shop_param_element} as e ON e.element_id=s.id"
			                          ." WHERE s.[act]='1' AND s.trash='0' AND e.param_id=%d AND e.value=%d AND s.site_id=%d"
			                          ." GROUP BY s.id"
			                          ." ORDER BY "
			                          .($this->diafan->sort ? $this->sort[$this->diafan->sort].',' : '')
			                          ." sort ASC, id ASC", $param_value["param_id"], $this->diafan->param, $this->diafan->cid,
			                          $this->diafan->_paginator->polog, $this->diafan->_paginator->nastr);

			$this->result["rows"] = $this->get_elements($result);

			$this->result["basket_link"] = $this->diafan->_route->module("basket");

			//сохранение кеша
			$this->diafan->_cache->save($this->result, $cache_meta, $this->diafan->module);
		}
		$this->result["buy"] = ! $this->diafan->configmodules('security_user', "shop") || $this->diafan->_user->id ? TRUE : FALSE;

		if ($this->result["rows"])
		{
			foreach ($this->result["rows"] as $i => $row)
			{
				$this->result["rows"][$i] = $this->select_price($row);

				$this->result["rows"][$i] = $this->useradmin($this->result["rows"][$i]);

				$this->result["rows"][$i]["tags"] = $this->diafan->configmodules("tags") ? $this->diafan->_tags->get($row["id"]) : '';

				$this->result["rows"][$i]["show_rating"] = $this->diafan->configmodules("rating") ? $this->diafan->_rating->get($row["id"]) : '';
			}
		}
		foreach ($this->sort as $key => $value)
		{
			$this->result["link_sort"][$key] =
			$this->diafan->sort != $key ? $this->diafan->_route->current_link("", array('sort' => $key)) : '';
		}
		$this->result["paginator"] = $this->diafan->_tpl->get('get', 'paginator', $this->result["paginator"]);
		return $this->result;
	}

	/**
	 * Генерирует данные для первой страницы магазина
	 * 
	 * @return array
	 */
	public function first_page()
	{
		if ($this->diafan->page)
		{
			include(ABSOLUTE_PATH.'includes/404.php');
		}
		//кеширование
		$cache_meta = array(
				"name"     => "first_page",
				"language" => _LANG,
				"site_id"  => $this->diafan->cid
			);
		if (! $this->result = $this->diafan->_cache->get($cache_meta, $this->diafan->module))
		{
			$this->result = array();

			$result = DB::query("SELECT id, [name], [anons], timeedit, site_id FROM {shop_category}"
			                    ." WHERE [act]='1' AND parent_id=0 AND trash='0' AND site_id=%d"
			                    ." ORDER by sort ASC, id ASC", $this->diafan->cid);

			while ($row = DB::fetch_array($result))
			{
				$this->result["categories"][$row["id"]]["name"] = $row["name"];
				$this->result["categories"][$row["id"]]["anons"] = $row["anons"];

				if (empty($this->result["timeedit"]) || $row["timeedit"] > $this->result["timeedit"])
				{
					$this->result["timeedit"] = $row["timeedit"];
				}

				$this->result["categories"][$row["id"]]["children"] = array();
				$result_children = DB::query("SELECT id, [name], [anons], site_id FROM {shop_category}"
				                             ." WHERE [act]='1' AND parent_id=%d AND trash='0' "
				                             ." AND site_id=%d ORDER BY sort ASC, id ASC",
				                             $row["id"], $this->diafan->cid);

				while ($child = DB::fetch_array($result_children))
				{
					$child["link"]  = $this->diafan->_route->link($child["site_id"], 'shop', $child["id"]);
					if ($this->diafan->configmodules("images_cat") && $this->diafan->configmodules("list_img_cat"))
					{
						$child["img"] = $this->diafan->_images->get($child["id"], $child["name"], $this->diafan->module,
						                                            $this->diafan->configmodules("list_img_cat") == 1 ? 1 : 0,
						                                            $child["link"], TRUE);
					}
					unset($child["site_id"]);
					$this->result["categories"][$row["id"]]["children"][] = $child;
				}

				$children = array();
				if ($this->diafan->configmodules("childrens_elements"))
				{
					$children = $this->diafan->get_children($row["id"],"shop_category");
				}
				$children[] = $row["id"];

				$result_img = DB::query_range("SELECT e.id, e.[name], e.cat_id, e.timeedit, e.[anons], e.site_id, e.no_buy, e.article FROM {shop} AS e"
				                              ." INNER JOIN {shop_category_rel} AS c ON e.id=c.element_id"
				                              ." WHERE c.cat_id IN (%s) AND e.[act]='1' AND e.trash='0'"
				                              ." GROUP BY e.id ORDER BY e.sort ASC, e.id ASC",
				                              implode(',', $children),
				                              0, $this->diafan->configmodules("count_list")
				                             );
				$this->result["categories"][$row["id"]]["rows"] = $this->get_elements($result_img);

				$this->result["categories"][$row["id"]]["link_all"] = $this->diafan->_route->link($row["site_id"], 'shop', $row["id"]);

				if ($this->diafan->configmodules("images_cat") && $this->diafan->configmodules("list_img_cat"))
				{
					$this->result["categories"][$row["id"]]["img"] =
					$this->diafan->_images->get($row["id"], $row["name"], $this->diafan->module,
					                            $this->diafan->configmodules("list_img_cat") == 1 ? 1 : 0,
					                            $this->result["categories"][$row["id"]]["link_all"],
					                            TRUE);
				}
			}

			$this->result["basket_link"] = $this->diafan->_route->module("basket");

			//сохранение кеша
			$this->diafan->_cache->save($this->result, $cache_meta, $this->diafan->module);
		}
		$this->result["buy"] = ! $this->diafan->configmodules('security_user', "shop") || $this->diafan->_user->id ? TRUE : FALSE;

		if (! empty($this->result["categories"]))
		{
			foreach ($this->result["categories"] as $cat => $rows)
			{
				if (! $rows)
					continue;

				$this->result["categories"][$cat]["name"] = $this->diafan->_useradmin->get($rows["name"], 'name', $cat, 'shop_category', _LANG);
				$this->result["categories"][$cat]["anons"] = $this->diafan->_useradmin->get($rows["anons"], 'anons', $cat, 'shop_category', _LANG);
				foreach ($rows["children"] as $k => $row)
				{
					$this->result["categories"][$cat]["children"][$k]["name"] =
						$this->diafan->_useradmin->get($row["name"], 'name', $row["id"], 'shop_category', _LANG);

					$this->result["categories"][$cat]["children"][$k]["anons"] =
						$this->diafan->_useradmin->get($row["anons"], 'anons', $row["id"], 'shop_category', _LANG);
				}

				foreach ($rows["rows"] as $k => $row)
				{
					$this->result["categories"][$cat]["rows"][$k] = $this->select_price($row);

					$this->result["categories"][$cat]["rows"][$k] = $this->useradmin($this->result["categories"][$cat]["rows"][$k]);

					$this->result["categories"][$cat]["rows"][$k]["tags"] = $this->diafan->configmodules("tags") ? $this->diafan->_tags->get($row["id"]) : '';

					$this->result["categories"][$cat]["rows"][$k]["show_rating"] =
						$this->diafan->configmodules("rating") ? $this->diafan->_rating->get($row["id"]) : '';
				}
			}
		}
		return $this->result;
	}

	/**
	 * Генерирует данные для списка товаров в группе
	 * 
	 * @return array
	 */
	public function list_category()
	{
		//кеширование
		$cache_meta = array(
				"name"     => "list",
				"cat_id"   => $this->diafan->cat,
				"language" => _LANG,
				"page"     => $this->diafan->page > 1 ? $this->diafan->page : 1,
				"site_id"  => $this->diafan->cid,
				"sort"     => $this->diafan->sort
			);
		if (! $this->result = $this->diafan->_cache->get($cache_meta, $this->diafan->module))
		{
			$this->result = array();

			$this->result["level"] = count($this->diafan->get_parents($this->diafan->cat, 'shop_category'));

			$this->result["path"] = $this->get_path('shop');

			$result = DB::query("SELECT id, [name], timeedit, [descr], [keywords], sort, parent_id, [title_meta] FROM {shop_category}"
			                    ." WHERE [act]='1' and id=%d AND trash='0' AND site_id=%d"
			                    ." ORDER BY sort ASC, id ASC",
			                    $this->diafan->cat, $this->diafan->cid);

			if (! $row = DB::fetch_array($result))
			{
				include_once(ABSOLUTE_PATH.'includes/404.php');
			}

			$this->result["id"]  = $row["id"];
			if ($this->diafan->configmodules("images_cat"))
			{
				$this->result["img"] = $this->diafan->_images->get($row["id"], $row["name"], $this->diafan->module, 0, 1, TRUE);
			}

			if ($this->diafan->configmodules("childrens_elements"))
			{
				$children = $this->diafan->get_children($this->diafan->cat, "shop_category");
			}
			$children[] = $this->diafan->cat;

			////navigation//
			$this->diafan->_paginator->page    = $this->diafan->page;
			$this->diafan->_paginator->navlink = $this->diafan->_route->current_link("page");
			$this->diafan->_paginator->navlink_tpl = $this->diafan->_route->current_link("", array("page" => "%d"));
			$this->diafan->_paginator->nen = DB::query_result("SELECT COUNT(DISTINCT e.id) FROM {shop} as e"
			                                                  ." INNER JOIN {shop_category_rel} AS c ON e.id=c.element_id"
			                                                  ." AND e.id=c.element_id AND c.cat_id IN (%s)"
			                                                  ." WHERE e.[act]='1' AND e.trash='0'",
			                                                  implode(',', $children)
			                                                 );
			$links = $this->diafan->_paginator->get();
			////navigation///

			$result_img = DB::query_range("SELECT s.id, s.[name], s.cat_id, s.timeedit, s.[anons], s.site_id, s.no_buy, s.article FROM {shop} AS s"
			                              .($this->diafan->sort == 1 || $this->diafan->sort == 2 ? " LEFT JOIN {shop_price} AS p ON p.good_id=s.id AND p.trash='0'" : '')
			                              ." INNER JOIN {shop_category_rel} AS c ON s.id=c.element_id AND c.cat_id IN (%s)"
			                              ." WHERE s.[act]='1' AND s.trash='0' GROUP BY s.id ORDER BY "
			                              .($this->diafan->sort ? $this->sort[$this->diafan->sort].',' : '')." s.sort ASC, s.id ASC",
			                              implode(',', $children),
			                              $this->diafan->_paginator->polog, $this->diafan->_paginator->nastr
			                             );

			$this->result["rows"] = $this->get_elements($result_img);

			$this->result["paginator"] = $links;

			$this->result["titlemodule"] = $row["name"];
			$this->result["edit_titlemodule"] = array("id" => $row["id"], "table" => "shop_category");
			$this->result["title_meta"]  = $row["title_meta"];
			$this->result["timeedit"]    = $row["timeedit"];
			$this->result["keywords"]    = $row["keywords"];
			$this->result["descr"]       = $row["descr"];

			$previous = DB::fetch_array(DB::query("SELECT [name], id FROM {shop_category} WHERE [act]='1' AND trash='0' AND site_id=%d"
			                                      ." AND (sort<%d OR sort=%d AND id<%d) AND parent_id=%d ORDER BY sort DESC, id DESC LIMIT 1",
			                                      $this->diafan->cid, $row["sort"], $row["sort"], $this->diafan->cat, $row["parent_id"]));
			if ($previous)
			{
				$this->result["previous"]["text"] = $previous["name"];
				$this->result["previous"]["id"]   = $previous["id"];
				$this->result["previous"]["link"] = $this->diafan->_route->link($this->diafan->cid, "shop", $previous["id"]);
			}
			$next = DB::fetch_array(DB::query("SELECT [name], id FROM {shop_category} WHERE [act]='1' AND trash='0' AND site_id=%d"
			                                  ." AND (sort>%d OR sort=%d AND id>%d) AND parent_id=%d ORDER BY sort ASC, id ASC LIMIT 1",
			                                  $this->diafan->cid, $row["sort"], $row["sort"], $this->diafan->cat, $row["parent_id"]));
			if ($next)
			{
				$this->result["next"]["text"] = $next["name"];
				$this->result["next"]["id"]   = $next["id"];
				$this->result["next"]["link"] = $this->diafan->_route->link($this->diafan->cid, "shop", $next["id"]);
			}

			$this->result["children"] = array();
			$result_children = DB::query("SELECT id, [name], [anons], site_id FROM {shop_category}"
			                             ." WHERE [act]='1' AND parent_id=%d AND trash='0'"
			                             ." AND site_id=%d ORDER BY sort ASC, id ASC",
			                             $row["id"], $this->diafan->cid);

			while ($child= DB::fetch_array($result_children))
			{
				$child["link"]  = $this->diafan->_route->link($child["site_id"], 'shop', $child["id"]);
				if ($this->diafan->configmodules("images_cat") && $this->diafan->configmodules("list_img_cat"))
				{
					$child["img"] = $this->diafan->_images->get($child["id"], $child["name"], $this->diafan->module,
																$this->diafan->configmodules("list_img_cat") == 1 ? 1 : 0,
																$child["link"], TRUE);
				}
				unset($child["site_id"]);
				$this->result["children"][] = $child;
			}

			$this->result["basket_link"] = $this->diafan->_route->module("basket");

			//сохранение кеша
			$this->diafan->_cache->save($this->result, $cache_meta, $this->diafan->module);
		}
		$this->result["buy"] = ! $this->diafan->configmodules('security_user', "shop") || $this->diafan->_user->id ? TRUE : FALSE;

		$text = DB::query_result("SELECT [text] FROM {shop_category} WHERE id=%d", $this->diafan->cat);
		$this->result["text"] = $this->diafan->_useradmin->get($text, 'text', $this->diafan->cat, 'shop_category', _LANG);

		if ($this->diafan->configmodules("cat") && $this->diafan->cat && $this->diafan->configmodules("comments_cat"))
		{
			$this->result["comments"] = $this->diafan->_comments->get();
		}

		if ($this->result["rows"])
		{
			foreach ($this->result["rows"] as $i => $row)
			{
				$this->result["rows"][$i] = $this->select_price($row);

				$this->result["rows"][$i] = $this->useradmin($this->result["rows"][$i]);

				$this->result["rows"][$i]["tags"] = $this->diafan->configmodules("tags") ? $this->diafan->_tags->get($row["id"]) : '';

				$this->result["rows"][$i]["show_rating"] = $this->diafan->configmodules("rating") ? $this->diafan->_rating->get($row["id"]) : '';
			}
			if (! empty($this->result["previous"]["text"]))
			{
				$this->result["previous"]["text"] =
					$this->diafan->_useradmin->get($this->result["previous"]["text"], 'name', $this->result["previous"]["id"], 'shop_category', _LANG);
			}
			if (! empty($this->result["next"]["text"]))
			{
				$this->result["next"]["text"] =
					$this->diafan->_useradmin->get($this->result["next"]["text"], 'name', $this->result["next"]["id"], 'shop_category', _LANG);
			}
			foreach ($this->result["children"] as $id => $row)
			{
				$this->result["children"][$id]["name"] = $this->diafan->_useradmin->get($row["name"], 'name', $row["id"], 'shop_category', _LANG);
				$this->result["children"][$id]["anons"] = $this->diafan->_useradmin->get($row["anons"], 'anons', $row["id"], 'shop_category', _LANG);
			}
			foreach ($this->result["path"] as $k => $path)
			{
				if ($k == 0)
					continue;
				$this->result["path"][$k]["name"] = $this->diafan->_useradmin->get($path["name"], 'name', $path["id"], 'shop_category', _LANG);
			}
		}

		foreach ($this->sort as $key => $value)
		{
			$this->result["link_sort"][$key] = $this->diafan->sort != $key ? $this->diafan->_route->current_link("", array('sort' => $key)) : '';
		}
		$this->result["paginator"] = $this->diafan->_tpl->get('get', 'paginator', $this->result["paginator"]);
		return $this->result;
	}

	/**
	 * Генерирует данные для страницы товара
	 * 
	 * @return array|boolean false
	 */
	public function id()
	{
		//кеширование
		$cache_meta = array(
				"name"     => "show",
				"cat_id"   => $this->diafan->cat,
				"show"     => $this->diafan->show,
				"language" => _LANG,
				"site_id"  => $this->diafan->cid
			);
		if (! $this->result = $this->diafan->_cache->get($cache_meta, $this->diafan->module))
		{
			$row = DB::fetch_array(DB::query("SELECT id, [name], timeedit, cat_id, [keywords], [descr], sort, [title_meta], no_buy, article FROM {shop}"
			                                 ." WHERE [act]='1' AND id = %d AND trash='0' AND site_id=%d",
			                                 $this->diafan->show, $this->diafan->cid));
			if (empty($row))
			{
				include(ABSOLUTE_PATH.'includes/404.php');
				return FALSE;
			}

			if (! $this->diafan->rewrite_module && ($this->diafan->configmodules("cat") && $this->diafan->cat != $row["cat_id"]
			   || ! $this->diafan->configmodules("cat") && $this->diafan->cat))
			{
				include(ABSOLUTE_PATH.'includes/404.php');
				return FALSE;
			}
			$this->diafan->cat  = $this->diafan->configmodules("cat") ? $row["cat_id"] : 0;
			$row["cat_id"]      = $this->diafan->configmodules("cat") ? $row["cat_id"] : 0;

			if ($this->diafan->configmodules("images"))
			{
				$this->result["img"] = $this->diafan->_images->get($row["id"], $row["name"], $this->diafan->module);
			}

			$row = $this->get_price($row);
			foreach ($row as $id => $value)
			{
				$this->result[$id] = $value;
			}
			$this->result["currency"]    = $this->diafan->configmodules("currency", "shop");
			$this->result["param"]       = $this->get_param($row["id"], $this->diafan->cid);
			$this->result["titlemodule"] = $row["name"];
			$this->result["edit_titlemodule"] = array("id" => $row["id"], "table" => "shop");

			if ($this->diafan->configmodules("count_rel_goods"))
			{
				$result_img = DB::query_range("SELECT s.id, s.[name], s.cat_id, s.timeedit, s.[anons], s.site_id, s.no_buy, s.article FROM {shop} AS s"
				                              ." INNER JOIN {shop_rel} AS r ON s.id=r.good_id AND r.rel_good_id=%d OR s.id=r.rel_good_id AND r.good_id=%d"
				                              ." WHERE s.[act]='1' AND s.trash='0' GROUP BY s.id",
				                              $this->diafan->show, $this->diafan->show,
				                              0, $this->diafan->configmodules("count_rel_goods")
				                             );
				$this->result["rel_goods"] = $this->get_elements($result_img);
			}

			$previous = DB::fetch_array(DB::query("SELECT [name], id, cat_id FROM {shop} WHERE [act]='1' AND trash='0' AND site_id=%d"
			                                      .($this->diafan->configmodules("cat") ? " AND cat_id='".$this->diafan->cat."'" : '')
			                                      ." AND (sort<%d OR sort=%d AND id<%d) ORDER BY sort DESC, id DESC LIMIT 1",
			                                      $this->diafan->cid, $row["sort"], $row["sort"], $this->diafan->show));
			if ($previous)
			{
				$this->result["previous"]["text"] = $previous["name"];
				if (! $this->diafan->configmodules("cat"))
				{
					$previous["cat_id"] = 0;
				}
				$this->result["previous"]["id"]   = $previous["id"];
				$this->result["previous"]["link"] = $this->diafan->_route->link($this->diafan->cid, "shop", $previous["cat_id"], $previous["id"]);
			}
			$next = DB::fetch_array(DB::query("SELECT [name], id, cat_id FROM {shop} WHERE [act]='1' AND trash='0' AND site_id=%d"
			                                  .($this->diafan->configmodules("cat") ? " AND cat_id='".$this->diafan->cat."'" : '')
			                                  ." AND (sort>%d OR sort=%d AND id>%d) ORDER BY sort ASC, id ASC LIMIT 1",
			                                  $this->diafan->cid, $row["sort"], $row["sort"], $this->diafan->show));
			if ($next)
			{
				$this->result["next"]["text"] = $next["name"];
				if (! $this->diafan->configmodules("cat"))
				{
					$next["cat_id"] = 0;
				}
				$this->result["next"]["id"]   = $next["id"];
				$this->result["next"]["link"] = $this->diafan->_route->link($this->diafan->cid, "shop", $next["cat_id"], $next["id"]);
			}

			$this->result["path"]        = $this->get_path('shop');

			$this->result["basket_link"] = $this->diafan->_route->module("basket");

			//сохранение кеша
			$this->diafan->_cache->save($this->result, $cache_meta, $this->diafan->module);
		}
		$this->diafan->cat = $this->result["cat_id"];

		$row = DB::fetch_array(DB::query("SELECT [text], [anons] FROM {shop} WHERE id=%d", $this->diafan->show));

		$this->result["text"]  = $row["text"];
		$this->result["anons"] = $row["anons"];

		if (! empty($this->result["previous"]["text"]))
		{
			$this->result["previous"]["text"] =
				$this->diafan->_useradmin->get($this->result["previous"]["text"], 'name', $this->result["previous"]["id"], 'shop', _LANG);
		}
		if (! empty($this->result["next"]["text"]))
		{
			$this->result["next"]["text"] =
				$this->diafan->_useradmin->get($this->result["next"]["text"], 'name', $this->result["next"]["id"], 'shop', _LANG);
		}
		foreach ($this->result["path"] as $k => $path)
		{
			if ($k == 0)
				continue;
			$this->result["path"][$k]["name"] = $this->diafan->_useradmin->get($path["name"], 'name', $path["id"], 'shop_category', _LANG);
		}

		$this->result["buy"]   = ! $this->diafan->configmodules('security_user', "shop") || $this->diafan->_user->id ? TRUE : FALSE;

		$this->result = $this->select_price($this->result);
		$this->result = $this->useradmin($this->result);

		if ($this->diafan->configmodules("rating"))
		{
			$this->result["show_rating"] = $this->diafan->_rating->get();
			$this->result["add_rating"] = $this->diafan->_rating->add();
		}

		if ($this->diafan->configmodules("comments"))
		{
			$this->result["comments"] = $this->diafan->_comments->get();
		}

		if ($this->diafan->configmodules("tags"))
		{
			$this->result["tags"] = $this->diafan->_tags->get();
		}
		if (! empty($this->result["rel_goods"]))
		{
			foreach ($this->result["rel_goods"] as $i => $row)
			{
				$this->result["rel_goods"][$i] = $this->select_price($row);
			}
		}

		return $this->result;
	}

	/**
	 * Генерирует данные для шаблонной функции: блок товаров
	 * 
	 * @param integer $count количество файлов
	 * @param integer $cat_id категория
	 * @param integer $site_id страница сайта
	 * @param string $sort сортировка date - по дате, rand - случайно
	 * @param integer $images количество фотографий
	 * @param string $param дополнительные параметры
	 * @return array
	 */
	public function show_block($count, $cat_id, $site_id, $sort, $images, $param)
	{
		//кеширование
		$cache_meta = array(
				"name"     => "block",
				"cat_id"   => $cat_id,
				"count"    => $count,
				"language" => _LANG,
				"site_id"  => $site_id,
				"images"   => $images,
				"param"    => $param,
				"sort"     => $sort
			);

		if ($sort == "rand" || ! $this->result = $this->diafan->_cache->get($cache_meta, "shop"))
		{
			if (! $site_id)
			{
				$site_id = DB::query_result("SELECT id FROM {site} WHERE addmodule='shop' AND trash='0' AND act='1'");
			}
			if ($cat_id && $this->diafan->configmodules("cat", "shop"))
			{
				if ($this->diafan->configmodules("childrens_elements", "shop"))
				{ 
					$children = $this->diafan->get_children($cat_id, "shop_category");
				}
				$children[] = $cat_id;
			}
			$params = array();
			if ($param)
			{
				$param = explode('&', $param);
				foreach ($param as $p)
				{
					list($id, $value) = explode('=', $p, 2);
					$id = preg_replace('/[^0-9]+/', '', $id);
					if (! empty($params[$id]))
					{
						if (is_array($params[$id]))
						{
							$params[$id][] = $value;
						}
						else
						{
							$params[$id] = array($params[$id], $value);
						}
					}
					else
					{
						$params[$id] = $value;
					}
				}
			}
			$where = '';
			$values = array();
			foreach ($params as $id => $value)
			{
				if (is_array($value))
				{
					$where .= "
					INNER JOIN {shop_param_element} AS pe".$id." ON pe".$id.".element_id=e.id AND pe".$id.".param_id='".$id."'"
					." AND pe".$id.".trash='0' AND (";
					foreach ($value as $val)
					{
						if ($value[0] != $val)
						{
							$where .= " OR ";
						}
						$where .= "pe".$id.".value='%h'";
						$values[] = $val;
					}
					$where .= ")";
				}
				else
				{
					$where .= "
					INNER JOIN {shop_param_element} AS pe".$id." ON pe".$id.".element_id=e.id AND pe".$id.".param_id='".$id."'"
					." AND pe".$id.".trash='0' AND pe".$id.".value='%h'";
					$values[] = $value;
				}
			}

			if ($sort == "rand")
			{
				$max_count = DB::query_result("SELECT COUNT(DISTINCT e.id) FROM {shop} as e"
				                              .($cat_id && $this->diafan->configmodules("cat","shop")
				                                ? " INNER JOIN {shop_category_rel} as c ON c.element_id=e.id"
				                                  ." AND c.cat_id IN (".implode(',', $children).")"
				                                : ''
				                               )
											  .$where
				                              ." WHERE e.[act]='1' AND e.trash='0'"
				                              .($site_id ? " AND e.site_id=".$site_id : ''),
											  $values
				                             );
				$rands = array();
				for ($i = 1; $i <= min($max_count, $count); $i++)
				{
					do
					{
						$rand = mt_rand(0, $max_count - 1);
					}
					while (in_array($rand, $rands));
					$rands[] = $rand;
				}
			}
			else
			{
				$rands[0] = 1;
			}
			$this->result["rows"] = array();

			foreach ($rands as $rand)
			{
				$result = DB::query("SELECT e.id, e.[name], e.[anons], e.cat_id, e.timeedit, e.site_id, e.no_buy, e.article FROM {shop} AS e"
				                    .($sort == "price" ? " LEFT JOIN {shop_price} AS p ON p.good_id=e.id AND p.trash='0'" : '')
				                    .($cat_id && $this->diafan->configmodules("cat", "shop")
				                      ? " INNER JOIN {shop_category_rel} as c ON c.element_id=e.id"
				                        ." AND c.cat_id IN (".implode(',', $children).")"
				                      : ''
				                     )
				                    .$where
				                    ." WHERE e.[act]='1' AND e.trash='0'"
				                    .($site_id ? " AND e.site_id=".$site_id : '')
				                    ." GROUP BY e.id"
				                    .($sort == "price" ? ' ORDER BY p.price ASC' : ($sort == "date" ? ' ORDER BY e.id DESC' : ''))
									.' LIMIT '
				                    .($sort == "rand"    ? $rand : 0).', '
				                    .($sort == "rand"    ? 1     : $count),
				                    $values);
				$this->result["rows"] = array_merge($this->result["rows"], $this->get_elements($result, $images, "block"));
			}

			if (! empty($this->result["rows"]) && $cat_id && $this->diafan->configmodules("cat", "shop"))
			{
				$cat = DB::fetch_array(DB::query("SELECT [name], site_id, id FROM {shop_category} WHERE id=%d", $cat_id));
				
				$this->result["name"] = $cat["name"];
				$this->result["link_all"] = $this->diafan->_route->link($cat["site_id"], 'shop', $cat["id"]);
				$this->result["category"] = TRUE;
			}
			elseif (! empty($this->result["rows"]) && $site_id)
			{
				$this->result["name"] = DB::query_result("SELECT [name] FROM {site} WHERE id=%d", $site_id);
				$this->result["link_all"] = $this->diafan->_route->link($site_id);
				$this->result["category"] = FALSE;
			}

			$this->result["basket_link"] = $this->diafan->_route->module("basket");

			//сохранение кеша
			if ($sort != "rand")
			{
				$this->diafan->_cache->save($this->result, $cache_meta, "shop");
			}
		}

		if ($this->result["rows"])
		{
			for ($i = 0; $i < count($this->result["rows"]); $i++)
			{
				$this->result["rows"][$i] = $this->select_price($this->result["rows"][$i]);
			}
		}

		if ($this->result["rows"])
		{
			foreach ($this->result["rows"] as $i => $row)
			{
				$this->result["rows"][$i] = $this->useradmin($this->result["rows"][$i]);
			}
		}
		$this->result["buy"] = ! $this->diafan->configmodules('security_user', "shop") || $this->diafan->_user->id ? TRUE : FALSE;
		return $this->result;
	}

	/**
	 * Генерирует данные для вывода тегов
	 * 
	 * @param integer $element_id  номер товара
	 * @return array
	 */
	public function tags($element_id)
	{
		$result = DB::query("SELECT id, [name], [anons], cat_id, timeedit, site_id, no_buy, article FROM {shop} WHERE [act]='1' AND trash='0' AND id=%d", $element_id);
		$this->result["rows"] = $this->get_elements($result, "list", "block");

		if (! $this->result["rows"])
		{
			return FALSE;
		}
		for ($i = 0; $i < count($this->result["rows"]); $i++)
		{
			$this->result["rows"][$i] = $this->select_price($this->result["rows"][$i]);
		}

		$this->result["basket_link"] = $this->diafan->_route->module("basket");

		$this->result["buy"] = ! $this->diafan->configmodules('security_user', "shop") || $this->diafan->_user->id ? TRUE : FALSE;
		return $this->result;
	}

	/**
	 * Генерирует контент для шаблонной функции: форма поиска по товарам
	 *
	 * @param integer|string $cat_id номер категории, в которой осуществляется поиск
	 * @param integer $site_id страница сайта, к которой прикреплен модуль Магазин
	 * @return array
	 */
	public function show_search($cat_id, $site_id)
	{
		$this->result = array();
		if (! $site_id)
		{
			if ($this->diafan->module == "shop")
			{
				$site_id = $this->diafan->cid;
			}
			else
			{
				$site_id = DB::query_result("SELECT id FROM {site} WHERE addmodule='shop' AND trash='0' AND act='1'");
			}
		}

		if ($cat_id === 'current')
		{
			if ($this->diafan->cat && $site_id == $this->diafan->cid)
			{
				$cat_id = $this->diafan->cat;
			}
			elseif (! empty($_GET["catid"]) && $site_id == $this->diafan->cid)
			{
				$this->diafan->cat = $this->diafan->get_param($_GET, "catid", '', 2);
				$cat_id = $this->diafan->cat;
			}
			else
			{
				$cat_id = 0;
			}
		}
		if (! $this->diafan->configmodules("cat", "shop", $site_id))
		{
			$cat_id = 0;
		}
		$this->result["path"]   = $this->diafan->_route->link($site_id);
		$this->result["cat_id"] = $cat_id;

		if ($this->diafan->configmodules("search_name", "shop", $site_id))
		{
			$this->result["name"] = array(
					"name"  => 1,
					"value" => ! empty($_GET["n"]) ? trim(htmlspecialchars(stripslashes($_GET["n"]))) : ''
				);
		}

		if ($this->diafan->configmodules("search_article", "shop", $site_id))
		{
			$this->result["article"] = array(
					"article"  => 1,
					"value" => ! empty($_GET["a"]) ? trim(htmlspecialchars(stripslashes($_GET["a"]))) : ''
				);
		}

		if ($this->diafan->configmodules("search_text", "shop", $site_id))
		{
			$this->result["text"] = array(
					"name"  => 1,
					"value" => ! empty($_GET["d"]) ? trim(htmlspecialchars(stripslashes($_GET["d"]))) : ''
				);
		}

		if ($this->diafan->configmodules("search_price", "shop", $site_id))
		{
			$this->result["price"] = array(
					"name"   => 1,
					"value1" => $this->diafan->get_param($_GET, "pr1", '', 2),
					"value2" => $this->diafan->get_param($_GET, "pr2", '', 2)
				);
		}

		$result = DB::query("SELECT DISTINCT(p.id), p.type, p.[name] FROM {shop_param} as p "
		                    .($this->diafan->configmodules("cat", "shop", $site_id)
		                      ? " INNER JOIN {shop_param_category_rel} AS c ON p.id=c.element_id AND c.cat_id"
		                        .($cat_id ? " IN (".$cat_id.", 0) " : '=0')
		                      : "")
		                    ." WHERE p.search='1' AND p.trash='0' AND p.site_id=%d ORDER BY p.sort ASC", $site_id);
		while ($row = DB::fetch_array($result))
		{
			if ($row["type"] == 'text' || $row["type"] == 'textarea' || $row["type"] == 'editor')
			{
				$row["value"] = ! empty($_GET["p".$row["id"]]) ? trim(htmlspecialchars(stripslashes($_GET["p".$row["id"]]))) : '';
			}
			elseif ($row["type"] == 'date')
			{
				$row["value1"] = $this->diafan->get_param($_GET, "p".$row["id"]."_1", '', 1);
				$row["value2"] = $this->diafan->get_param($_GET, "p".$row["id"]."_2", '', 1);
			}
			elseif ($row["type"] == 'numtext')
			{
				$row["value1"] = $this->diafan->get_param($_GET, "p".$row["id"]."_1", '', 2);
				$row["value2"] = $this->diafan->get_param($_GET, "p".$row["id"]."_2", '', 2);
			}
			elseif ($row["type"] == 'checkbox')
			{
				$row["value"] = $this->diafan->get_param($_GET, "p".$row["id"], '', 2);
			}
			elseif ($row["type"] == 'select' || $row["type"] == 'multiple')
			{
				$result_select = DB::query("SELECT [name], id FROM {shop_param_select} WHERE param_id=%d ORDER BY sort ASC", $row["id"]);
				while ($row_select = DB::fetch_array($result_select))
				{
					$row["select_array"][$row_select["id"]] = $row_select["name"];
				}
				if (! empty($_GET["p".$row["id"]]) && ! is_array($_GET["p".$row["id"]]))
				{
					$row["value"][] = $this->diafan->get_param($_GET, "p".$row["id"], 0, 2);
				}
				elseif (! empty($_GET["p".$row["id"]]) && is_array($_GET["p".$row["id"]]))
				{
					foreach ($_GET["p".$row["id"]] as $val)
					{
						$row["value"][] = intval($val);
					}
				}
				else
				{
					$row["value"] = array();
				}
			}
			unset($row["param"]);
			$this->result["rows"][] = $row;
		}
		return $this->result;
	}

	/**
	 * Форматирует данные о товаре для списка товаров
	 *
	 * @param resource $result результат выполнения SQL-запроса
	 * @param string $images показывать фотографии
	 * @param string $function функция, для которой генерируется список товаров
	 * @return array
	 */
	public function get_elements($result, $images = 'list', $function = 'list')
	{
		if (empty($this->result["timeedit"]))
		{
			$this->result["timeedit"] = '';
		}
		$rows = array();
		while ($row = DB::fetch_array($result))
		{
			if (! $this->diafan->configmodules("cat", "shop", $row["site_id"]))
			{
				$row["cat_id"] = 0;
			}
			if ($row["timeedit"] < $this->result["timeedit"])
			{
				$this->result["timeedit"] = $row["timeedit"];
			}
			unset($row["timeedit"]);

			$row["link"] = $this->diafan->_route->link($row["site_id"], "shop", $row["cat_id"], $row["id"]);

			if ($this->diafan->configmodules("images", "shop", $row["site_id"]))
			{
				if ($images === 'list' && $this->diafan->configmodules("list_img", "shop", $row["site_id"]))
				{
					$row["img"]  = $this->diafan->_images->get($row["id"], $row["name"], 'shop',
					               $this->diafan->configmodules("list_img", "shop", $row["site_id"]) == 1 ? 1 : 0,
					               $row["link"]);
				}
				elseif ($images != 'list' && $images)
				{
					$row["img"]  = $this->diafan->_images->get($row["id"], $row["name"], 'shop', $images, $row["link"]);
				}
			}
			$row = $this->get_price($row);
			$row["param"] = $this->get_param($row["id"], $row["site_id"], $function);

			$rows[] = $row;	
		}
		$this->result["currency"] = $this->diafan->configmodules("currency", "shop");
		return $rows;
	}

	/**
	 * Формирует данные о цене товара
	 *
	 * @param array $row данные о товаре
	 * @return array
	 */
	private function get_price($row)
	{
		$row["param_select_buy"] = array();
		$params = array();

		$result = DB::query("SELECT e.param_id, e.value FROM {shop_param_element} as e"
		                    ." INNER JOIN {shop_param} as p ON e.param_id=p.id"
		                    ." LEFT JOIN {shop_param_select} as s ON e.param_id=s.param_id AND e.value=s.id"
		                    ." WHERE e.element_id=%d AND p.`type`='multiple' AND p.required='1'"
		                    ." GROUP BY e.id ORDER BY p.sort, s.sort ASC", $row["id"]);
		while ($row_param = DB::fetch_array($result))
		{
			$param["value"] = $row_param["value"];

			$param["selected"] = FALSE;
			$row["param_select_buy"][$row_param["param_id"]][] = $param;

			if (empty($this->result["depends"][$row_param["param_id"]]))
			{
				$this->result["depends"][$row_param["param_id"]]["name"] = DB::query_result("SELECT [name] FROM {shop_param} WHERE id=%d", $row_param["param_id"]);
				$result_select = DB::query("SELECT [name], id FROM {shop_param_select} WHERE param_id=%d", $row_param["param_id"]);
				while ($row_select = DB::fetch_array($result_select))
				{
					$this->result["depends"][$row_param["param_id"]]["values"][$row_select["id"]] = $row_select["name"];
				}
			}
		}
		foreach ($row["param_select_buy"] as $id => $rows)
		{
			if (count($row["param_select_buy"][$id]) < 2)
			{
				unset($row["param_select_buy"][$id]);
			}
		}

		$row["depend"] = array();
		$row["price"] = 0;
		$row["price_id"] = 0;
		$row["count"] = $this->diafan->configmodules("use_count_goods", "shop") ? 0 : TRUE;
		$pr1 = $this->diafan->get_param($_GET, "pr1", "", 2);
		$pr2 = $this->diafan->get_param($_GET, "pr2", "", 2);
		$result = DB::query("SELECT p.price, p.param_id, p.param_value, p.count_goods, p.id FROM {shop_price} as p"
							." LEFT JOIN {shop_param_select} as s ON p.param_id=s.param_id AND p.param_value=s.id"
							." WHERE p.good_id=%d AND p.trash='0'".($pr1 ? " AND p.price>=".$pr1 : '').($pr2 ? " AND p.price<=".$pr2 : '')
		                    ." GROUP BY p.id ORDER BY s.sort ASC", $row["id"]);
		while ($row_param = DB::fetch_array($result))
		{
			// если задана цена без зависимых параметров, то цена, считается независимой
			if (! $row_param["param_id"])
			{
				$row["price"]    = $row_param["price"];
				$row["price_id"] = $row_param["id"];
				if ($this->diafan->configmodules("use_count_goods", "shop"))
				{
					$row["count"] = $row_param["count_goods"];
				}
				$row["depend"] = array();
				return $row;
			}
			if (! empty($row["param_select_buy"][$row_param["param_id"]]))
			{
				unset($row["param_select_buy"][$row_param["param_id"]]);
			}
			$row["depend"]["param_id"] = $row_param["param_id"];
			$depend["price"]    = $row_param["price"];
			$depend["price_id"] = $row_param["id"];
			$depend["value"]    = $row_param["param_value"];
			$depend["count"]    = ! $this->diafan->configmodules("use_count_goods", "shop") || $row_param["count_goods"];
			$row["count"] = TRUE;
			$depend["selected"] = FALSE;
			$row["depend"]["rows"][] = $depend;
			if (empty($this->result["depends"][$row_param["param_id"]]))
			{
				$this->result["depends"][$row_param["param_id"]]["name"] = DB::query_result("SELECT [name] FROM {shop_param} WHERE id=%d", $row_param["param_id"]);
				$result_select = DB::query("SELECT [name], id FROM {shop_param_select} WHERE param_id=%d ORDER BY sort ASC", $row_param["param_id"]);
				while ($row_select = DB::fetch_array($result_select))
				{
					$this->result["depends"][$row_param["param_id"]]["values"][$row_select["id"]] = $row_select["name"];
				}
			}
		}

		return $row;
	}

	/**
	 * Задает выделение параметров, учитываемый при покупке товара
	 *
	 * @param array $row данные о товаре
	 * @return array
	 */
	private function select_price($row)
	{
		$row["discount"] = $this->get_discount($row);
		if ($row["discount"] && $row["price_id"])
		{
			$row["old_price"] = $this->diafan->_useradmin->get($this->diafan->format_price($row["price"]), 'price', $row["price_id"], 'shop_price', '', 'text');
			$row["price"] = $this->diafan->format_price($row["price"] * (100 - $row["discount"]) / 100);
		}
		elseif ($row["price_id"])
		{
			$row["price"] = $this->diafan->_useradmin->get($this->diafan->format_price($row["price"]), 'price', $row["price_id"], 'shop_price', '', 'text');
		}

		$row["selected"] = FALSE;
		if (! empty($row["param_select_buy"]))
		{
			foreach ($row["param_select_buy"] as $id => $params)
			{
				$new_params = array();
				foreach ($params as $param)
				{
					if (! empty($_GET["p".$id]) && $_GET["p".$id] == $param["value"])
					{
						$param["selected"] = TRUE;
					}
					$new_params[] = $param;
				}
				$row["param_select_buy"][$id] = $new_params;
			}
		}
		if (! empty($row["depend"]["rows"]))
		{
			$new_params = array();
			foreach ($row["depend"]["rows"] as $param)
			{
				if ($row["discount"])
				{
					$param["old_price"] = $this->diafan->_useradmin->get($param["price"], 'price', $param["price_id"], 'shop_price', '', 'text');
					$param["price"] = $param["price"] * (100 - $row["discount"]) / 100;
				}
				else
				{
					$param["price"] = $this->diafan->_useradmin->get($this->diafan->format_price($param["price"]), 'price', $param["price_id"], 'shop_price', '', 'text');
				}
				if (! empty($_GET["p".$row["depend"]["param_id"]]) && in_array($param["value"], $_GET["p".$row["depend"]["param_id"]]))
				{
					$param["selected"] = TRUE;
					if ($row["discount"])
					{
						$row["old_price"] = $param["old_price"];
						$row["price"]     = $param["price"];
					}
					$row["count"]      = $param["count"];
					$row["selected"]   = TRUE;
				}
				$param["count_in_basket"] = 0;
				if (! empty($_SESSION["basket"][$row["id"]]))
				{
					foreach ($_SESSION["basket"][$row["id"]] as $key => $basket)
					{
						if (strstr($key, '|depend='.$param["value"].'|'))
						{
							$param["count_in_basket"] += $basket["count"];
						}
					}
				}
				$new_params[] = $param;
			}
			$row["depend"]["rows"] = $new_params;
		}
		else
		{
			if (! empty($_SESSION["basket"][$row["id"]]))
			{
				$row["count_in_basket"] = 0;
				foreach ($_SESSION["basket"][$row["id"]] as $basket)
				{
					$row["count_in_basket"] += $basket["count"];
				}
			}
		}

		return $row;
	}

	/**
	 * Определяет скидку на товар
	 *
	 * @param array $row данные о товаре
	 * @return float
	 */
	private function get_discount($row)
	{
		// выбираем все скидки, актуальные на текущий момент
		if (! isset($this->discounts["discounts"]))
		{
			$this->discounts["discounts"] = array();
			$result = DB::query("SELECT id, discount FROM {shop_discount} WHERE trash='0' AND act='1' AND (date_start='0' OR date_start<=%d) AND (date_finish='0' OR date_finish>=%d)", time(), time());
			while ($dis = DB::fetch_array($result))
			{
				$this->discounts["discounts"][$dis["id"]] = $dis["discount"];
			}
		}
		if ($this->discounts["discounts"])
		{
			// проверяем установлена ли скидка на товар
			if (! isset($this->discounts["good".$row["id"]]))
			{
				$this->discounts["good".$row["id"]] = DB::query_result("SELECT discount_id FROM {shop_discount_object} WHERE good_id=%d AND discount_id IN (%h) ORDER BY id ASC LIMIT 1", $row["id"], implode(",", array_keys($this->discounts["discounts"])));
			}
			if ($this->discounts["good".$row["id"]])
			{
				return $this->discounts["discounts"][$this->discounts["good".$row["id"]]];
			}
			// проверяем установлена ли скидка на категорию
			if ($row["cat_id"] && ! isset($this->discounts["cat".$row["cat_id"]]))
			{
				$this->discounts["cat".$row["cat_id"]] = DB::query_result("SELECT discount_id FROM {shop_discount_object} WHERE cat_id=%d AND discount_id IN (%h) ORDER BY id ASC LIMIT 1", $row["cat_id"], implode(",", array_keys($this->discounts["discounts"])));
			}
			if ($row["cat_id"] && $this->discounts["cat".$row["cat_id"]])
			{
				return $this->discounts["discounts"][$this->discounts["cat".$row["cat_id"]]];
			}
			// проверяем установлена ли скидка на все товары
			if (! isset($this->discounts["all"]))
			{
				$this->discounts["all"] = DB::query_result("SELECT discount_id FROM {shop_discount_object} WHERE cat_id=0 AND good_id=0 AND discount_id IN (%h) ORDER BY id ASC LIMIT 1", implode(",", array_keys($this->discounts["discounts"])));
			}
			if ($this->discounts["all"])
			{
				return $this->discounts["discounts"][$this->discounts["all"]];
			}
		}
		return 0;
	}

	/**
	 * Получает дополнительные характеристики товара
	 * 
	 * @param integer $id номер товара
	 * @param integer $site_id номер страницы, к которой прикреплен товар
	 * @param string $function функция, для которой выбираются параметры
	 * @return array
	 */
	private function get_param($id, $site_id, $function = "id")
	{
		global $param_select, $param_select_page;
		$values    = array();
		$rvalues   = array();
		$result_el = DB::query("SELECT e.value as rvalue, e.[value], e.param_id, e.id FROM {shop_param_element} as e"
							   ." LEFT JOIN {shop_param_select} as s ON e.param_id=s.param_id AND e.param_id=s.id"
							   ." WHERE e.element_id=%d GROUP BY e.id ORDER BY s.sort ASC", $id);
		while ($row_el = DB::fetch_array($result_el))
		{
			$values[$row_el["param_id"]][]  = $row_el;
		}
		$result = DB::query("SELECT p.id, p.[name], p.type, p.page FROM {shop_param} as p "
		                    .($this->diafan->configmodules("cat", "shop", $site_id)
		                      ? " INNER JOIN {shop_category_rel} as c ON c.element_id=".$id
		                        ." INNER JOIN {shop_param_category_rel} as cp ON (cp.cat_id=c.cat_id OR cp.cat_id=0) AND cp.element_id=p.id"
		                      : "")
		                    ." WHERE p.trash='0' AND p.site_id=%d"
		                    .($function == "block" ? " AND p.block='1'" : '')
		                    .($function == "list"  ? " AND p.list='1'"  : '')
		                    ." GROUP BY p.id ORDER BY p.sort ASC", $site_id
		                   );

		$param = array();
		while ($row = DB::fetch_array($result))
		{
			switch ($row["type"])
			{
				case "text":
				case "textarea":
				case "editor":
					if (! empty($values[$row["id"]][0]["value"]))
					{
						$param[] = array(
							"id" => $row["id"],
							"name" => $row["name"],
							"value" => $values[$row["id"]][0]["value"],
							"value_id" => $values[$row["id"]][0]["id"],
							"type" => $row["type"]
						);
					}
					break;
				case "date":
					if (! empty($values[$row["id"]][0]["rvalue"]))
					{
						$param[] = array(
							"id" => $row["id"],
							"name" => $row["name"],
							"value" => date("d.m.Y", $values[$row["id"]][0]["rvalue"]),
							"value_id" => $values[$row["id"]][0]["id"],
							"type" => $row["type"]
						);
					}
					break;
				case "select":
					$value = ! empty($values[$row["id"]][0]["rvalue"]) ? $values[$row["id"]][0]["rvalue"] : '';
					if ($value)
					{
						if (empty($param_select[$row["id"]][$value]))
						{
							$param_select[$row["id"]][$value] = DB::query_result("SELECT [name] FROM {shop_param_select} WHERE id=%d AND param_id=%d", $values[$row["id"]][0]["rvalue"], $row["id"]);
						}
						if ($row["page"])
						{
							if (empty($param_select_page[$row["id"]][$value]))
							{
								$param_select_page[$row["id"]][$value] = $this->diafan->_route->link($site_id, "shop", 0, 0, $value);
							}
							$link  = $param_select_page[$row["id"]][$value];
							$param[] = array("id" => $row["id"], "name" => $row["name"], "value" => $param_select[$row["id"]][$value], "link" => $link);
						}
						else
						{
							$param[] = array("id" => $row["id"], "name" => $row["name"], "value" => $param_select[$row["id"]][$value]);
						}
					}
					break;
				case "multiple":
					if (! empty($values[$row["id"]]))
					{
						$value = array();
						foreach ($values[$row["id"]] as $val)
						{
							if (empty($param_select[$row["id"]][$val["rvalue"]]))
							{
								$param_select[$row["id"]][$val["rvalue"]] =
								DB::query_result("SELECT [name] FROM {shop_param_select} WHERE id=%d AND param_id=%d", $val["rvalue"], $row["id"]);
							}
							if ($row["page"])
							{
								if ($this->diafan->module == 'shop' && $this->diafan->param == $val["rvalue"])
								{
									$link = '';
								}
								else
								{
									if (empty($param_select_page[$row["id"]][$val["rvalue"]]))
									{
										$param_select_page[$row["id"]][$val["rvalue"]] = $this->diafan->_route->link($site_id, "shop", 0, 0, $val["rvalue"]);
									}
									$link = $param_select_page[$row["id"]][$val["rvalue"]];
								}
								$value[] = array("id" => $row["id"], "name" => $param_select[$row["id"]][$val["rvalue"]], "link" => $link);
							}
							else
							{
								$value[] = $param_select[$row["id"]][$val["rvalue"]];
							}
						}
						$param[] = array("id" => $row["id"], "name" => $row["name"], "value" => $value);
					}
					break;
				case "checkbox":
					$value = ! empty($values[$row["id"]][0]["rvalue"]) ? 1 : 0;
					if (! isset($param_select[$row["id"]][$value]))
					{
						$param_select[$row["id"]][$value] =
						DB::query_result("SELECT [name] FROM {shop_param_select} WHERE value=%d AND param_id=%d", $value, $row["id"]);
					}
					if (! $param_select[$row["id"]][$value] && $value == 1)
					{
						$param[] = array("id" => $row["id"], "name" => $row["name"], "value" => '');
					}
					else
					{
						$param[] = array("id" => $row["id"], "name" => $row["name"], "value" => $param_select[$row["id"]][$value]);
					}
					break;
				case "title":
					$param[] = array(
						"id" => $row["id"],
						"name" => $row["name"],
						"value" => ''
					);
					break;
				default:
					if (! empty($values[$row["id"]][0]["rvalue"]))
					{
						$param[] = array(
							"id" => $row["id"],
							"name" => $row["name"],
							"value" => $values[$row["id"]][0]["rvalue"],
							"value_id" => $values[$row["id"]][0]["id"],
							"type" => $row["type"]
						);
					}
					break;
			}
		}
		return $param;
	}

	/**
	 * Формирует SQL-запрос при поиске по товарам
	 * 
	 * @return array
	 */
	private function get_where()
	{
		$where_param  = '';
		$where        = ' AND s.site_id=%d';
		$values       = array();
		$values_param = array();
		$getnav       = '';

		if (isset($_GET["catid"]))
		{
			$this->diafan->cat = $this->diafan->get_param($_GET,  "catid", '', 2);
			$catarr = array(0);
			$getnav = '?catid='.$this->diafan->cat;
			if ($this->diafan->cat)
			{
				if ($this->diafan->configmodules("childrens_elements"))
				{
					$children = $this->diafan->get_children($this->diafan->cat, "shop_category");
				}
				$where_param .= " INNER JOIN {shop_category_rel} AS c ON s.id=c.element_id AND "
				                .($this->diafan->configmodules("childrens_elements")
				                  ? 'c.cat_id in ('.implode(',', $children).')'
				                  : 'c.cat_id='.$this->diafan->cat);
			}

			if (! empty($_GET["n"]) && $this->diafan->configmodules("search_name"))
			{
				$where   .= " AND s.name"._LANG." LIKE '%%%h%%'";
				$_GET["n"] = trim($this->diafan->get_param($_GET,  "n", '', 1));
				$values[] = $_GET["n"];
				$getnav  .= '&n='.$_GET["n"];
			}

			if (! empty($_GET["a"]) && $this->diafan->configmodules("search_article"))
			{
				$where   .= " AND s.article"._LANG."='%h'";
				$_GET["a"] = trim($this->diafan->get_param($_GET,  "a", '', 1));
				$values[] = $_GET["a"];
				$getnav  .= '&a='.$_GET["a"];
			}

			if (! empty($_GET["d"]) && $this->diafan->configmodules("search_text"))
			{
				$where   .= " AND s.text"._LANG." LIKE '%%%h%%'";
				$_GET["d"] = trim($this->diafan->get_param($_GET,  "d", '', 1));
				$values[] = $_GET["d"];
				$getnav  .= '&d='.$_GET["d"];
			}

			if (! empty($_GET["pr1"]) || ! empty($_GET["pr2"]))
			{
				$where_param .= " INNER JOIN {shop_price} AS pr ON pr.good_id=s.id AND pr.trash='0'"
				                .(! empty($_GET["pr1"]) ? " AND pr.price>=%d" : '')
				                .(! empty($_GET["pr2"]) ? " AND pr.price<=%d" : '');
				if (! empty($_GET["pr1"]))
				{
					$pr1 = $this->diafan->get_param($_GET,  "pr1", '', 2);
					$values_param[] = $pr1;
					$getnav .= '&pr1='.$pr1;
				}
				if (! empty($_GET["pr2"]))
				{
					$pr2 = $this->diafan->get_param($_GET,  "pr2", '', 2);
					$values_param[] = $pr2;
					$getnav .= '&pr2='.$pr2;
				}
			}

			$result = DB::query("SELECT DISTINCT(p.id), p.type FROM {shop_param} as p "
			                    .($this->diafan->configmodules("cat")
			                      ? " INNER JOIN {shop_param_category_rel} AS c ON p.id=c.element_id AND c.cat_id"
			                        .($this->diafan->cat ? " IN (".$this->diafan->cat.", 0) " : '=0')
			                      : "")
			                    ." WHERE p.search='1' AND p.trash='0' AND p.site_id=%d", $this->diafan->cid);
			while ($row = DB::fetch_array($result))
			{
				if (($row["type"] == 'text' || $row["type"] == 'textarea' || $row["type"] == 'editor') && ! empty($_GET["p".$row["id"]]))
				{
					$val = trim($this->diafan->get_param($_GET, "p".$row["id"], '', 1));
					$where_param .= "
					INNER JOIN {shop_param_element} AS pe".$row["id"]." ON pe".$row["id"].".element_id=s.id AND pe".$row["id"].".param_id='%d'"
					." AND pe".$row["id"].".trash='0' AND pe".$row["id"].".value"._LANG." LIKE '%%%h%%'";
					$values_param[] = $row["id"];
					$values_param[] = $val;
					$getnav .= '&p'.$row["id"].'='.$val;
				}
				elseif ($row["type"] == 'date' && (! empty($_GET["p".$row["id"]."_1"]) || ! empty($_GET["p".$row["id"]."_2"])))
				{
					$where_param .= "
					INNER JOIN {shop_param_element} AS pe".$row["id"]." ON pe".$row["id"].".element_id=s.id AND pe".$row["id"].".param_id='%d'"
					." AND pe".$row["id"].".trash='0'";
					$values_param[] = $row["id"];
					if (! empty($_GET["p".$row["id"]."_1"]))
					{
						$where_param .= " AND pe".$row["id"].".value>=%d";
						$values_param[] = $this->diafan->unixdate($_GET["p".$row["id"]."_1"]);
						$getnav .= '&p'.$row["id"].'_1='.trim($this->diafan->get_param($_GET, "p".$row["id"]."_1", '', 1));
					}
					if (! empty($_GET["p".$row["id"]."_2"]))
					{
						$where_param .= " AND pe".$row["id"].".value<=%d";
						$values_param[] = $this->diafan->unixdate($_GET["p".$row["id"]."_2"]);
						$getnav .= '&p'.$row["id"].'_2='.trim($this->diafan->get_param($_GET, "p".$row["id"]."_2", '', 1));
					}
				}
				elseif ($row["type"] == 'numtext' && (! empty($_GET["p".$row["id"]."_2"]) ||  ! empty($_GET["p".$row["id"]."_1"])))
				{
					$val1 = $this->diafan->get_param($_GET, "p".$row["id"]."_1", '', 2);
					$val2 = $this->diafan->get_param($_GET, "p".$row["id"]."_2", '', 2);
					$where_param .= "
					INNER JOIN {shop_param_element} AS pe".$row["id"]." ON pe".$row["id"].".element_id=s.id AND pe".$row["id"].".param_id='%d'"
					." AND pe".$row["id"].".trash='0'"
					.($val1 ? " AND pe".$row["id"].".value>=%d" : '')
					.($val2 ? " AND pe".$row["id"].".value<=%d" : '')
					;
					$values_param[] = $row["id"];
					if ($val1)
					{
						$values_param[] = $val1;
						$getnav .= '&p'.$row["id"].'_1='.$val1;
					}
					if ($val2)
					{
						$values_param[] = $val2;
						$getnav .= '&p'.$row["id"].'_2='.$val2;
					}
				}
				elseif ($row["type"] == 'checkbox' && ! empty($_GET["p".$row["id"]]))
				{
					$where_param .= "
					INNER JOIN {shop_param_element} AS pe".$row["id"]." ON pe".$row["id"].".element_id=s.id AND pe".$row["id"].".param_id='%d'"
					." AND pe".$row["id"].".trash='0' AND pe".$row["id"].".value='1'";
					$values_param[] = $row["id"];
					$getnav .= '&p'.$row["id"].'=1';
				}
				elseif (($row["type"] == 'select' || $row["type"] == 'multiple') && ! empty($_GET["p".$row["id"]]))
				{
					if (! is_array($_GET["p".$row["id"]]))
					{
						$val = $this->diafan->get_param($_GET, "p".$row["id"], '', 2);
						$where_param .= "
						INNER JOIN {shop_param_element} AS pe".$row["id"]." ON pe".$row["id"].".element_id=s.id AND pe".$row["id"].".param_id='%d'"
						." AND pe".$row["id"].".trash='0' AND pe".$row["id"].".value='%d'";
						$values_param[] = $row["id"];
						$values_param[] = $val;
						$getnav .= '&p'.$row["id"].'='.$val;
					}
					else
					{
						$vals= array();
						foreach ($_GET["p".$row["id"]] as $val)
						{
							if ($val)
							{
								$val = intval($val);
								$vals[] = $val;
								$getnav .= '&p'.$row["id"].'[]='.$val;
							}
						}
						if (! empty($vals))
						{
							$where_param .= "
							INNER JOIN {shop_param_element} AS pe".$row["id"]." ON pe".$row["id"].".element_id=s.id AND pe".$row["id"].".param_id='%d'"
							." AND pe".$row["id"].".trash='0' AND pe".$row["id"].".value IN (".implode(", ", $vals).")";
							$values_param[] = $row["id"];
							if (! empty($_GET["pr1"]) || ! empty($_GET["pr2"]))
							{
								$where_param .= " AND (pr.param_id<>pe".$row["id"].".param_id OR pr.param_value=pe".$row["id"].".value)";
							}
						}
					}
				}
			}
		}
		$values = array_merge($values_param, array($this->diafan->cid), $values);
		return array($where, $where_param, $values, $getnav);
	}

	/**
	 * Генерирует ссылки на форму редактирования
	 *
	 * @param array $row данные о товаре
	 * @return array
	 */
	private function useradmin($row)
	{
		if (! empty($row["name"]))
		{
			$row["name"] = $this->diafan->_useradmin->get($row["name"], 'name', $row["id"], 'shop', _LANG);
		}
		if (! empty($row["article"]))
		{
			$row["article"] = $this->diafan->_useradmin->get($row["article"], 'article', $row["id"], 'shop', '', 'text');
		}
		if (! empty($row["text"]))
		{
			$row["text"] = $this->diafan->_useradmin->get($row["text"], 'text', $row["id"], 'shop', _LANG);
		}

		$row["anons"] = $this->diafan->_useradmin->get($row["anons"], 'anons', $row["id"], 'shop',  _LANG);

		foreach ($row["param"] as $k => $param)
		{
			$row["param"][$k]["name"] = $this->diafan->_useradmin->get($param["name"], 'name', $param["id"], 'shop_param');
			if (! empty($param["value_id"]))
			{
				$lang = in_array($param["type"], array('text', 'textarea', 'editor')) ? _LANG : '';
				$row["param"][$k]["value"] = $this->diafan->_useradmin->get($param["value"], 'value', $param["value_id"], 'shop_param_element', $lang, $param["type"]);
			}
		}

		return $row;
	}
}